/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 2004 g10 Code GmbH
 *
 * Parts were writte/modified by:
 * Nico Golde <nico@ngolde.de>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#include "sort.h"
#include "mutt.h"

#ifndef _MAKEDOC
#define DT_MASK         0x0f
#define DT_BOOL         1       /* boolean option */
#define DT_NUM          2       /* a number */
#define DT_STR          3       /* a string */
#define DT_PATH         4       /* a pathname */
#define DT_QUAD         5       /* quad-option (yes/no/ask-yes/ask-no) */
#define DT_SORT         6       /* sorting methods */
#define DT_RX           7       /* regular expressions */
#define DT_MAGIC        8       /* mailbox type */

#define DTYPE(x) ((x) & DT_MASK)

/* subtypes */
#define DT_SUBTYPE_MASK 0xf0
#define DT_SORT_ALIAS   0x10
#define DT_SORT_BROWSER 0x20
#define DT_SORT_KEYS    0x40
#define DT_SORT_AUX     0x80

/* flags to parse_set() */
#define M_SET_INV       (1<<0)  /* default is to invert all vars */
#define M_SET_UNSET     (1<<1)  /* default is to unset all vars */
#define M_SET_RESET     (1<<2)  /* default is to reset all vars to default */

struct option_t {
  const char *option;
  short type;
  unsigned long data;
  const char *init;
};

#define UL (unsigned long)

#endif /* _MAKEDOC */

/* build complete documentation */

struct option_t MuttVars[] = {
  /*++*/
  {"abort_noattach", DT_QUAD, OPT_ATTACH, "no" },
  /*
   ** .pp
   ** This variable specifies whether to abort sending if no attachment
   ** was made but the content references them, i.e. the content
   ** matches the regular expression given in
   ** $$attach_remind_regexp. If a match was found and this
   ** variable is set to \fIyes\fP, message sending will be aborted
   ** but the mail will be send nevertheless if set to \fIno\fP.
   **
   ** .pp
   ** This variable and $$attach_remind_regexp are intended to
   ** remind the user to attach files if the message's text
   ** references them.
   **
   ** .pp
   ** See also the $$attach_remind_regexp variable.
   */
  {"abort_nosubject", DT_QUAD, OPT_SUBJECT, "ask-yes" },
  /*
   ** .pp
   ** If set to \fIyes\fP, when composing messages and no subject is given
   ** at the subject prompt, composition will be aborted.  If set to
   ** \fIno\fP, composing messages with no subject given at the subject
   ** prompt will never be aborted.
   */
  {"abort_unmodified", DT_QUAD, OPT_ABORT, "yes" },
  /*
   ** .pp
   ** If set to \fIyes\fP, composition will automatically abort after
   ** editing the message body if no changes are made to the file (this
   ** check only happens after the \fIfirst\fP edit of the file).  When set
   ** to \fIno\fP, composition will never be aborted.
   */
  {"allow_8bit", DT_BOOL, OPTALLOW8BIT, "yes" },
  /*
   ** .pp
   ** Controls whether 8-bit data is converted to 7-bit using either
   ** \fTquoted-printable\fP or \fTbase64\fP encoding when sending mail.
   */
  {"askbcc", DT_BOOL, OPTASKBCC, "no" },
  /*
   ** .pp
   ** If \fIset\fP, Madmutt will prompt you for blind-carbon-copy (Bcc) recipients
   ** before editing an outgoing message.
   */
  {"askcc", DT_BOOL, OPTASKCC, "no" },
  /*
   ** .pp
   ** If \fIset\fP, Madmutt will prompt you for carbon-copy (Cc) recipients before
   ** editing the body of an outgoing message.
   */
  {"attach_format", DT_STR, UL &AttachFormat, "%u%D%I %t%4n %T%.40d%> [%.7m/%.10M, %.6e%?C?, %C?, %s] "},
  /*
   ** .pp
   ** This variable describes the format of the ``attachment'' menu.  The
   ** following \fTprintf(3)\fP-style sequences are understood:
   ** .pp
   ** .dl
   ** .dt %C  .dd charset
   ** .dt %c  .dd requires charset conversion (n or c)
   ** .dt %D  .dd deleted flag
   ** .dt %d  .dd description
   ** .dt %e  .dd MIME \fTContent-Transfer-Encoding:\fP header field
   ** .dt %f  .dd filename
   ** .dt %I  .dd MIME \fTContent-Disposition:\fP header field (\fTI\fP=inline, \fTA\fP=attachment)
   ** .dt %m  .dd major MIME type
   ** .dt %M  .dd MIME subtype
   ** .dt %n  .dd attachment number
   ** .dt %Q  .dd "Q", if MIME part qualifies for attachment counting
   ** .dt %s  .dd size
   ** .dt %t  .dd tagged flag
   ** .dt %T  .dd graphic tree characters
   ** .dt %u  .dd unlink (=to delete) flag
   ** .dt %X  .dd number of qualifying MIME parts in this part and its children
   ** .dt %>X .dd right justify the rest of the string and pad with character "X"
   ** .dt %|X .dd pad to the end of the line with character "X"
   ** .de
   */
  {"attach_remind_regexp", DT_RX, UL &AttachRemindRegexp, "attach"},
  /*
   ** .pp
   ** If this variable is non-empty, madmutt will scan a message's contents
   ** before sending for this regular expression. If it is found, it will
   ** ask for what to do depending on the setting of $$abort_noattach.
   ** .pp
   ** This variable and $$abort_noattach are intended to remind the user
   ** to attach files if the message's text references them.
   */
  {"attach_sep", DT_STR, UL &AttachSep, "\n"},
  /*
   ** .pp
   ** The separator to add between attachments when operating (saving,
   ** printing, piping, etc) on a list of tagged attachments.
   */
  {"attach_split", DT_BOOL, OPTATTACHSPLIT, "yes" },
  /*
   ** .pp
   ** If this variable is \fIunset\fP, when operating (saving, printing, piping,
   ** etc) on a list of tagged attachments, Madmutt will concatenate the
   ** attachments and will operate on them as a single attachment. The
   ** ``$$attach_sep'' separator is added after each attachment. When \fIset\fP,
   ** Madmutt will operate on the attachments one by one.
   */
  {"attribution", DT_STR, UL &Attribution, "On %d, %n wrote:"},
  /*
   ** .pp
   ** This is the string that will precede a message which has been included
   ** in a reply.  For a full listing of defined \fTprintf(3)\fP-like sequences see
   ** the section on ``$$index_format''.
   */
  {"autoedit", DT_BOOL, OPTAUTOEDIT, "no" },
  /*
   ** .pp
   ** When \fIset\fP along with ``$$edit_headers'', Madmutt will skip the initial
   ** send-menu and allow you to immediately begin editing the body of your
   ** message.  The send-menu may still be accessed once you have finished
   ** editing the body of your message.
   ** .pp
   ** Also see ``$$fast_reply''.
   */
  {"auto_tag", DT_BOOL, OPTAUTOTAG, "no" },
  /*
   ** .pp
   ** When \fIset\fP, functions in the \fIindex\fP menu which affect a message
   ** will be applied to all tagged messages (if there are any).  When
   ** unset, you must first use the ``tag-prefix'' function (default: "\fT;\fP") to
   ** make the next function apply to all tagged messages.
   */
  {"bounce", DT_QUAD, OPT_BOUNCE, "ask-yes" },
  /*
   ** .pp
   ** Controls whether you will be asked to confirm bouncing messages.
   ** If set to \fIyes\fP you don't get asked if you want to bounce a
   ** message. Setting this variable to \fIno\fP is not generally useful,
   ** and thus not recommended, because you are unable to bounce messages.
   */
  {"bounce_delivered", DT_BOOL, OPTBOUNCEDELIVERED, "yes" },
  /*
   ** .pp
   ** When this variable is \fIset\fP, Madmutt will include
   ** \fTDelivered-To:\fP header fields when bouncing messages.
   ** Postfix users may wish to \fIunset\fP this variable.
   */
  {"check_new", DT_BOOL, OPTCHECKNEW, "yes" },
  /*
   ** .pp
   ** \fBNote:\fP this option only affects \fImaildir\fP and \fIMH\fP style
   ** mailboxes.
   ** .pp
   ** When \fIset\fP, Madmutt will check for new mail delivered while the
   ** mailbox is open.  Especially with MH mailboxes, this operation can
   ** take quite some time since it involves scanning the directory and
   ** checking each file to see if it has already been looked at.  If it's
   ** \fIunset\fP, no check for new mail is performed while the mailbox is open.
   */
  {"collapse_unread", DT_BOOL, OPTCOLLAPSEUNREAD, "yes" },
  /*
   ** .pp
   ** When \fIunset\fP, Madmutt will not collapse a thread if it contains any
   ** unread messages.
   */
  {"uncollapse_jump", DT_BOOL, OPTUNCOLLAPSEJUMP, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will jump to the next unread message, if any,
   ** when the current thread is \fIun\fPcollapsed.
   */
  {"compose_format", DT_STR, UL &ComposeFormat, "-- Madmutt: Compose  [Approx. msg size: %l   Atts: %a]%>-"},
  /*
   ** .pp
   ** Controls the format of the status line displayed in the ``compose''
   ** menu.  This string is similar to ``$$status_format'', but has its own
   ** set of \fTprintf(3)\fP-like sequences:
   ** .pp
   ** .dl
   ** .dt %a .dd total number of attachments
   ** .dt %h .dd local hostname
   ** .dt %l .dd approximate size (in bytes) of the current message
   ** .dt %v .dd Madmutt version string
   ** .de
   ** .pp
   ** See the text describing the ``$$status_format'' option for more
   ** information on how to set ``$$compose_format''.
   */
  {"config_charset", DT_STR, UL &ConfigCharset, "" },
  /*
   ** .pp
   ** When defined, Madmutt will recode commands in rc files from this
   ** encoding.
   */
  {"confirmappend", DT_BOOL, OPTCONFIRMAPPEND, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will prompt for confirmation when appending messages to
   ** an existing mailbox.
   */
  {"confirmcreate", DT_BOOL, OPTCONFIRMCREATE, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will prompt for confirmation when saving messages to a
   ** mailbox which does not yet exist before creating it.
   */
  {"connect_timeout", DT_NUM, UL &ConnectTimeout, "30" },
  /*
   ** .pp
   ** Causes Madmutt to timeout a network connection (for IMAP or POP) after this
   ** many seconds if the connection is not able to be established.  A negative
   ** value causes Madmutt to wait indefinitely for the connection to succeed.
   */
  {"content_type", DT_STR, UL &ContentType, "text/plain"},
  /*
   ** .pp
   ** Sets the default \fTContent-Type:\fP header field for the body
   ** of newly composed messages.
   */
  {"copy", DT_QUAD, OPT_COPY, "yes" },
  /*
   ** .pp
   ** This variable controls whether or not copies of your outgoing messages
   ** will be saved for later references.  Also see ``$$record'',
   ** and ``$fcc-hook''.
   */
  {"date_format", DT_STR, UL &DateFmt, "!%a, %b %d, %Y at %I:%M:%S%p %Z"},
  /*
   ** .pp
   ** This variable controls the format of the date printed by the ``\fT%d\fP''
   ** sequence in ``$$index_format''.  This is passed to \fTstrftime(3)\fP
   ** to process the date.
   ** .pp
   ** Unless the first character in the string is a bang (``\fT!\fP''), the month
   ** and week day names are expanded according to the locale specified in
   ** the variable ``$$locale''. If the first character in the string is a
   ** bang, the bang is discarded, and the month and week day names in the
   ** rest of the string are expanded in the \fIC\fP locale (that is in US
   ** English).
   */
  {"default_hook", DT_STR, UL &DefaultHook, "~f %s !~P | (~P ~C %s)"},
  /*
   ** .pp
   ** This variable controls how send-hooks, message-hooks, save-hooks,
   ** and fcc-hooks will
   ** be interpreted if they are specified with only a simple regexp,
   ** instead of a matching pattern.  The hooks are expanded when they are
   ** declared, so a hook will be interpreted according to the value of this
   ** variable at the time the hook is declared.  The default value matches
   ** if the message is either from a user matching the regular expression
   ** given, or if it is from you (if the from address matches
   ** ``alternates'') and is to or cc'ed to a user matching the given
   ** regular expression.
   */
  {"delete", DT_QUAD, OPT_DELETE, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not messages are really deleted when closing or
   ** synchronizing a mailbox.  If set to \fIyes\fP, messages marked for
   ** deleting will automatically be purged without prompting.  If set to
   ** \fIno\fP, messages marked for deletion will be kept in the mailbox.
   */
  {"delete_space", DT_BOOL, OPTDELSP, "no" },
  /*
   ** .pp
   ** When sending messages with \fTformat=flowed\fP by \fIsetting\fP the
   ** $$text_flowed variable, this variable specifies whether to also
   ** set the \fTDelSp\fP parameter to \fTyes\fP. If this is \fIunset\fP,
   ** no additional parameter will be send as a value of \fTno\fP already
   ** is the default behavior.
   **
   ** .pp
   ** \fBNote:\fP this variable only has an effect on \fIoutgoing\fP messages
   ** (if $$text_flowed is \fIset\fP) but not on incomming.
   */
  {"delete_untag", DT_BOOL, OPTDELETEUNTAG, "yes" },
  /*
   ** .pp
   ** If this option is \fIset\fP, Madmutt will untag messages when marking them
   ** for deletion.  This applies when you either explicitly delete a message,
   ** or when you save it to another folder.
   */
  {"digest_collapse", DT_BOOL, OPTDIGESTCOLLAPSE, "yes" },
  /*
   ** .pp
   ** If this option is \fIset\fP, Madmutt's received-attachments menu will not show the subparts of
   ** individual messages in a multipart/digest.  To see these subparts, press 'v' on that menu.
   */
  {"display_filter", DT_PATH, UL &DisplayFilter, ""},
  /*
   ** .pp
   ** When \fIset\fP, specifies a command used to filter messages.  When a message
   ** is viewed it is passed as standard input to $$display_filter, and the
   ** filtered message is read from the standard output.
   */
  {"duplicate_threads", DT_BOOL, OPTDUPTHREADS, "yes" },
  /*
   ** .pp
   ** This variable controls whether Madmutt, when sorting by threads, threads
   ** messages with the same \fTMessage-ID:\fP header field together.
   ** If it is \fIset\fP, it will indicate that it thinks they are duplicates
   ** of each other with an equals sign in the thread diagram.
   */
  {"edit_headers", DT_BOOL, OPTEDITHDRS, "no" },
  /*
   ** .pp
   ** This option allows you to edit the header of your outgoing messages
   ** along with the body of your message.
   **
   ** .pp
   ** Which empty header fields to show is controlled by the
   ** $$editor_headers option.
   */
#ifdef USE_NNTP
  {"editor_headers", DT_STR, UL &EditorHeaders, "From: To: Cc: Bcc: Subject: Reply-To: Newsgroups: Followup-To: X-Comment-To:" },
#else
  {"editor_headers", DT_STR, UL &EditorHeaders, "From: To: Cc: Bcc: Subject: Reply-To:" },
#endif
  /*
   ** .pp
   ** If $$edit_headers is \fIset\fP, this space-separated list specifies
   ** which \fInon-empty\fP header fields to edit in addition to
   ** user-defined headers.
   **
   ** .pp
   ** Note: if $$edit_headers had to be turned on by force because
   ** $$strict_mailto is \fIunset\fP, this option has no effect.
   */
  {"encode_from", DT_BOOL, OPTENCODEFROM, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will \fTquoted-printable\fP encode messages when
   ** they contain the string ``\fTFrom \fP'' (note the trailing space)
   ** in the beginning of a line. Useful to avoid the tampering certain mail
   ** delivery and transport agents tend to do with messages.
   **
   ** .pp
   ** \fBNote:\fP as Madmutt currently violates RfC3676 defining
   ** \fTformat=flowed\fP, it's <em/strongly/ advised to \fIset\fP
   ** this option although discouraged by the standard. Alternatively,
   ** you must take care of space-stuffing <tt/From / lines (with a trailing
   ** space) yourself.
   */
  {"fast_reply", DT_BOOL, OPTFASTREPLY, "no" },
  /*
   ** .pp
   ** When \fIset\fP, the initial prompt for recipients and subject are skipped
   ** when replying to messages, and the initial prompt for subject is
   ** skipped when forwarding messages.
   ** .pp
   ** \fBNote:\fP this variable has no effect when the ``$$autoedit''
   ** variable is \fIset\fP.
   */
  {"fcc_attach", DT_BOOL, OPTFCCATTACH, "yes" },
  /*
   ** .pp
   ** This variable controls whether or not attachments on outgoing messages
   ** are saved along with the main body of your message.
   */
  {"fcc_clear", DT_BOOL, OPTFCCCLEAR, "no" },
  /*
   ** .pp
   ** When this variable is \fIset\fP, FCCs will be stored unencrypted and
   ** unsigned, even when the actual message is encrypted and/or
   ** signed.
   ** (PGP only)
   */
  {"folder", DT_PATH, UL &Maildir, "~/Mail"},
  /*
   ** .pp
   ** Specifies the default location of your mailboxes.  A ``\fT+\fP'' or ``\fT=\fP'' at the
   ** beginning of a pathname will be expanded to the value of this
   ** variable.  Note that if you change this variable from the default
   ** value you need to make sure that the assignment occurs \fIbefore\fP
   ** you use ``+'' or ``='' for any other variables since expansion takes place
   ** during the ``set'' command.
   */
  {"folder_format", DT_STR, UL &FolderFormat, "%2C %t %N %F %2l %-8.8u %-8.8g %8s %d %f"},
  /*
   ** .pp
   ** This variable allows you to customize the file browser display to your
   ** personal taste.  This string is similar to ``$$index_format'', but has
   ** its own set of \fTprintf(3)\fP-like sequences:
   ** .pp
   ** .dl
   ** .dt %C  .dd current file number
   ** .dt %d  .dd date/time folder was last modified
   ** .dt %f  .dd filename
   ** .dt %F  .dd file permissions
   ** .dt %g  .dd group name (or numeric gid, if missing)
   ** .dt %l  .dd number of hard links
   ** .dt %N  .dd N if folder has new mail, blank otherwise
   ** .dt %s  .dd size in bytes
   ** .dt %t  .dd * if the file is tagged, blank otherwise
   ** .dt %u  .dd owner name (or numeric uid, if missing)
   ** .dt %>X .dd right justify the rest of the string and pad with character "X"
   ** .dt %|X .dd pad to the end of the line with character "X"
   ** .de
   */
  {"followup_to", DT_BOOL, OPTFOLLOWUPTO, "yes" },
  /*
   ** .pp
   ** Controls whether or not the \fTMail-Followup-To:\fP header field is
   ** generated when sending mail.  When \fIset\fP, Madmutt will generate this
   ** field when you are replying to a known mailing list, specified with
   ** the ``subscribe'' or ``$lists'' commands or detected by common mailing list
   ** headers.
   ** .pp
   ** This field has two purposes.  First, preventing you from
   ** receiving duplicate copies of replies to messages which you send
   ** to mailing lists. Second, ensuring that you do get a reply
   ** separately for any messages sent to known lists to which you are
   ** not subscribed.  The header will contain only the list's address
   ** for subscribed lists, and both the list address and your own
   ** email address for unsubscribed lists.  Without this header, a
   ** group reply to your message sent to a subscribed list will be
   ** sent to both the list and your address, resulting in two copies
   ** of the same email for you.
   */
  {"force_buffy_check", DT_BOOL, OPTFORCEBUFFYCHECK, "no" },
  /*
   ** .pp
   ** When \fIset\fP, it causes Madmutt to check for new mail when the
   ** \fIbuffy-list\fP command is invoked. When \fIunset\fP, \fIbuffy_list\fP
   ** will just list all mailboxes which are already known to have new mail.
   ** .pp
   ** Also see the following variables: ``$$timeout'', ``$$mail_check'' and
   ** ``$$imap_mail_check''.
   */
  {"forward_decode", DT_BOOL, OPTFORWDECODE, "yes" },
  /*
   ** .pp
   ** Controls the decoding of complex MIME messages into \fTtext/plain\fP when
   ** forwarding a message.  The message header is also RFC2047 decoded.
   ** This variable is only used, if ``$$mime_forward'' is \fIunset\fP,
   ** otherwise ``$$mime_forward_decode'' is used instead.
   */
  {"forward_edit", DT_QUAD, OPT_FORWEDIT, "yes" },
  /*
   ** .pp
   ** This quadoption controls whether or not the user is automatically
   ** placed in the editor when forwarding messages.  For those who always want
   ** to forward with no modification, use a setting of \fIno\fP.
   */
  {"forward_format", DT_STR, UL &ForwFmt, "[%a: %s]"},
  /*
   ** .pp
   ** This variable controls the default subject when forwarding a message.
   ** It uses the same format sequences as the ``$$index_format'' variable.
   */
  {"forward_quote", DT_BOOL, OPTFORWQUOTE, "no" },
  /*
   ** .pp
   ** When \fIset\fP forwarded messages included in the main body of the
   ** message (when ``$$mime_forward'' is \fIunset\fP) will be quoted using
   ** ``$$indent_string''.
   */
  {"header", DT_BOOL, OPTHEADER, "no" },
  /*
   ** .pp
   ** When \fIset\fP, this variable causes Madmutt to include the header
   ** of the message you are replying to into the edit buffer.
   ** The ``$$weed'' setting applies.
   */
  {"hidden_host", DT_BOOL, OPTHIDDENHOST, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will skip the host name part of ``$$hostname'' variable
   ** when adding the domain part to addresses.  This variable does not
   ** affect the generation of \fTMessage-ID:\fP header fields, and it will not lead to the
   ** cut-off of first-level domains.
   */
  {"hide_limited", DT_BOOL, OPTHIDELIMITED, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not show the presence of messages that are hidden
   ** by limiting, in the thread tree.
   */
  {"hide_missing", DT_BOOL, OPTHIDEMISSING, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not show the presence of missing messages in the
   ** thread tree.
   */
  {"hide_thread_subject", DT_BOOL, OPTHIDETHREADSUBJECT, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not show the subject of messages in the thread
   ** tree that have the same subject as their parent or closest previously
   ** displayed sibling.
   */
  {"hide_top_limited", DT_BOOL, OPTHIDETOPLIMITED, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not show the presence of messages that are hidden
   ** by limiting, at the top of threads in the thread tree.  Note that when
   ** $$hide_missing is \fIset\fP, this option will have no effect.
   */
  {"hide_top_missing", DT_BOOL, OPTHIDETOPMISSING, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not show the presence of missing messages at the
   ** top of threads in the thread tree.  Note that when $$hide_limited is
   ** \fIset\fP, this option will have no effect.
   */
  {"history", DT_NUM, UL &HistSize, "10" },
  /*
   ** .pp
   ** This variable controls the size (in number of strings remembered) of
   ** the string history buffer. The buffer is cleared each time the
   ** variable is changed.
   */
  {"honor_followup_to", DT_QUAD, OPT_MFUPTO, "yes" },
  /*
   ** .pp
   ** This variable controls whether or not a \fTMail-Followup-To:\fP header field is
   ** honored when group-replying to a message.
   */
  {"ignore_list_reply_to", DT_BOOL, OPTIGNORELISTREPLYTO, "no" },
  /*
   ** .pp
   ** Affects the behaviour of the \fIreply\fP function when replying to
   ** messages from mailing lists.  When \fIset\fP, if the ``\fTReply-To:\fP'' header field is
   ** set to the same value as the ``\fTTo:\fP'' header field, Madmutt assumes that the
   ** ``\fTReply-To:\fP'' header field was set by the mailing list to automate responses
   ** to the list, and will ignore this field.  To direct a response to the
   ** mailing list when this option is set, use the \fIlist-reply\fP
   ** function; \fIgroup-reply\fP will reply to both the sender and the
   ** list.
   ** Remember: This option works only for mailing lists which are explicitly set in your madmuttrc
   ** configuration file.
   */
  {"imap_authenticators", DT_STR, UL &ImapAuthenticators, "" },
  /*
   ** .pp
   ** This is a colon-delimited list of authentication methods Madmutt may
   ** attempt to use to log in to an IMAP server, in the order Madmutt should
   ** try them.  Authentication methods are either ``\fTlogin\fP'' or the right
   ** side of an IMAP ``\fTAUTH=\fP'' capability string, e.g. ``\fTdigest-md5\fP'',
   ** ``\fTgssapi\fP'' or ``\fTcram-md5\fP''. This parameter is case-insensitive.
   ** .pp
   ** If this
   ** parameter is \fIunset\fP (the default) Madmutt will try all available methods,
   ** in order from most-secure to least-secure.
   ** .pp
   ** Example: \fTset imap_authenticators="gssapi:cram-md5:login"\fP
   ** .pp
   ** \fBNote:\fP Madmutt will only fall back to other authentication methods if
   ** the previous methods are unavailable. If a method is available but
   ** authentication fails, Madmutt will not connect to the IMAP server.
   */
  { "imap_check_subscribed",  DT_BOOL, OPTIMAPCHECKSUBSCRIBED, "no" },
  /*
   ** .pp
   ** When \fIset\fP, mutt will fetch the set of subscribed folders from
   ** your server on connection, and add them to the set of mailboxes
   ** it polls for new mail. See also the ``$mailboxes'' command.
   */
  {"imap_delim_chars", DT_STR, UL &ImapDelimChars, "/."},
  /*
   ** .pp
   ** This contains the list of characters which you would like to treat
   ** as folder separators for displaying IMAP paths. In particular it
   ** helps in using the '\fT=\fP' shortcut for your $$folder variable.
   */
  {"imap_headers", DT_STR, UL &ImapHeaders, "" },
  /*
   ** .pp
   ** Madmutt requests these header fields in addition to the default headers
   ** (``DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE
   ** CONTENT-DESCRIPTION IN-REPLY-TO REPLY-TO LINES X-LABEL'') from IMAP
   ** servers before displaying the ``index'' menu. You may want to add more
   ** headers for spam detection.
   ** .pp
   ** \fBNote:\fP This is a space separated list.
   */
  {"imap_keepalive", DT_NUM, UL &ImapKeepalive, "900" },
  /*
   ** .pp
   ** This variable specifies the maximum amount of time in seconds that Madmutt
   ** will wait before polling open IMAP connections, to prevent the server
   ** from closing them before Madmutt has finished with them.
   ** .pp
   ** The default is
   ** well within the RFC-specified minimum amount of time (30 minutes) before
   ** a server is allowed to do this, but in practice the RFC does get
   ** violated every now and then.
   ** .pp
   ** Reduce this number if you find yourself
   ** getting disconnected from your IMAP server due to inactivity.
   */
  {"imap_login", DT_STR, UL &ImapLogin, "" },
  /*
   ** .pp
   ** Your login name on the IMAP server.
   ** .pp
   ** This variable defaults to the value of ``$$imap_user.''
   */
  {"imap_list_subscribed", DT_BOOL, OPTIMAPLSUB, "no" },
  /*
   ** .pp
   ** This variable configures whether IMAP folder browsing will look for
   ** only subscribed folders or all folders.  This can be toggled in the
   ** IMAP browser with the \fItoggle-subscribed\fP function.
   */
  {"imap_mail_check", DT_NUM, UL &ImapBuffyTimeout, "300" },
  /*
   ** .pp
   ** This variable configures how often (in seconds) Madmutt should look for
   ** new mail in IMAP folders. This is split from the ``$mail_check'' variable
   ** to generate less traffic and get more accurate information for local folders.
   */
  {"imap_pass", DT_STR, UL &ImapPass, "" },
  /*
   ** .pp
   ** Specifies the password for your IMAP account.  If \fIunset\fP, Madmutt will
   ** prompt you for your password when you invoke the fetch-mail function.
   ** .pp
   ** \fBWarning\fP: you should only use this option when you are on a
   ** fairly secure machine, because the superuser can read your configuration even
   ** if you are the only one who can read the file.
   */
  {"imap_passive", DT_BOOL, OPTIMAPPASSIVE, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will not open new IMAP connections to check for new
   ** mail.  Madmutt will only check for new mail over existing IMAP
   ** connections.  This is useful if you don't want to be prompted to
   ** user/password pairs on Madmutt invocation, or if opening the connection
   ** is slow.
   */
  {"imap_reconnect", DT_QUAD, OPT_IMAPRECONNECT, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not Madmutt will try to reconnect to IMAP server when
   ** the connection is lost.
   */
  {"imap_servernoise", DT_BOOL, OPTIMAPSERVERNOISE, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will display warning messages from the IMAP
   ** server as error messages. Since these messages are often
   ** harmless, or generated due to configuration problems on the
   ** server which are out of the users' hands, you may wish to suppress
   ** them at some point.
   */
  {"imap_user", DT_STR, UL &ImapUser, "" },
  /*
   ** .pp
   ** The name of the user whose mail you intend to access on the IMAP
   ** server.
   ** .pp
   ** This variable defaults to your user name on the local machine.
   */
  {"implicit_autoview", DT_BOOL, OPTIMPLICITAUTOVIEW, "no" },
  /*
   ** .pp
   ** If \fIset\fP, Madmutt will look for a mailcap entry with the
   ** ``\fTcopiousoutput\fP'' flag set for \fIevery\fP MIME attachment it doesn't have
   ** an internal viewer defined for.  If such an entry is found, Madmutt will
   ** use the viewer defined in that entry to convert the body part to text
   ** form.
   */
  {"include", DT_QUAD, OPT_INCLUDE, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not a copy of the message(s) you are replying to
   ** is included in your reply.
   */
  {"indent_string", DT_STR, UL &Prefix, "> "},
  /*
   ** .pp
   ** Specifies the string to prepend to each line of text quoted in a
   ** message to which you are replying.  You are strongly encouraged not to
   ** change this value, as it tends to agitate the more fanatical netizens.
   */
  {"index_format", DT_STR, UL &HdrFmt, "%4C %Z %{%b %d} %-15.15L (%?l?%4l&%4c?) %s"},
  /*
   ** .pp
   ** This variable allows you to customize the message index display to
   ** your personal taste.
   ** .pp
   ** ``Format strings'' are similar to the strings used in the ``C''
   ** function \fTprintf(3)\fP to format output (see the man page for more detail).
   ** The following sequences are defined in Madmutt:
   ** .pp
   ** .dl
   ** .dt %a .dd address of the author
   ** .dt %A .dd reply-to address (if present; otherwise: address of author)
   ** .dt %b .dd filename of the original message folder (think mailBox)
   ** .dt %B .dd the list to which the letter was sent, or else the folder name (%b).
   ** .dt %c .dd number of characters (bytes) in the message
   ** .dt %C .dd current message number
   ** .dt %d .dd date and time of the message in the format specified by
   **            ``date_format'' converted to sender's time zone
   ** .dt %D .dd date and time of the message in the format specified by
   **            ``date_format'' converted to the local time zone
   ** .dt %e .dd current message number in thread
   ** .dt %E .dd number of messages in current thread
   ** .dt %f .dd entire From: line (address + real name)
   ** .dt %F .dd author name, or recipient name if the message is from you
   ** .dt %H .dd spam attribute(s) of this message
   ** .dt %g .dd newsgroup name (if compiled with nntp support)
   ** .dt %i .dd message-id of the current message
   ** .dt %l .dd number of lines in the message (does not work with maildir,
   **            mh, and possibly IMAP folders)
   ** .dt %L .dd If an address in the To or CC header field matches an address
   **            defined by the users ``subscribe'' command, this displays
   **            "To <list-name>", otherwise the same as %F.
   ** .dt %m .dd total number of message in the mailbox
   ** .dt %M .dd number of hidden messages if the thread is collapsed.
   ** .dt %N .dd message score
   ** .dt %n .dd author's real name (or address if missing)
   ** .dt %O .dd (_O_riginal save folder)  Where Madmutt would formerly have
   **            stashed the message: list name or recipient name if no list
   ** .dt %s .dd subject of the message
   ** .dt %S .dd status of the message (N/D/d/!/r/\(as)
   ** .dt %t .dd `to:' field (recipients)
   ** .dt %T .dd the appropriate character from the $$to_chars string
   ** .dt %u .dd user (login) name of the author
   ** .dt %v .dd first name of the author, or the recipient if the message is from you
   ** .dt %W .dd name of organization of author (`organization:' field)
   ** .dt %X .dd number of attachments
   ** .dt %y .dd `x-label:' field, if present
   ** .dt %Y .dd `x-label' field, if present, and (1) not at part of a thread tree,
   **            (2) at the top of a thread, or (3) `x-label' is different from
   **            preceding message's `x-label'.
   ** .dt %Z .dd message status flags
   ** .dt %{fmt} .dd the date and time of the message is converted to sender's
   **                time zone, and ``fmt'' is expanded by the library function
   **                ``strftime''; a leading bang disables locales
   ** .dt %[fmt] .dd the date and time of the message is converted to the local
   **                time zone, and ``fmt'' is expanded by the library function
   **                ``strftime''; a leading bang disables locales
   ** .dt %(fmt) .dd the local date and time when the message was received.
   **                ``fmt'' is expanded by the library function ``strftime'';
   **                a leading bang disables locales
   ** .dt %<fmt> .dd the current local time. ``fmt'' is expanded by the library
   **                function ``strftime''; a leading bang disables locales.
   ** .dt %>X    .dd right justify the rest of the string and pad with character "X"
   ** .dt %|X    .dd pad to the end of the line with character "X"
   ** .de
   ** .pp
   ** See also: ``$$to_chars''.
   */
  {"keep_flagged", DT_BOOL, OPTKEEPFLAGGED, "no" },
  /*
   ** .pp
   ** If \fIset\fP, read messages marked as flagged will not be moved
   ** from your spool mailbox to your ``$$mbox'' mailbox, or as a result of
   ** a ``$mbox-hook'' command.
   */
  {"locale", DT_STR, UL &Locale, "C"},
  /*
   ** .pp
   ** The locale used by \fTstrftime(3)\fP to format dates. Legal values are
   ** the strings your system accepts for the locale variable \fTLC_TIME\fP.
   */
  {"force_list_reply", DT_QUAD, OPT_LISTREPLY, "ask-no" },
  /*
   ** .pp
   ** This variable controls what effect ordinary replies have on mailing list
   ** messages: if \fIset\fP, a normal reply will be interpreted as list-reply
   ** while if it's \fIunset\fP the reply functions work as usual.
   */
  {"max_display_recips", DT_NUM, UL &MaxDispRecips, "0" },
  /*
   ** .pp
   ** When set non-zero, this specifies the maximum number of recipient header
   ** lines (\fTTo:\fP, \fTCc:\fP and \fTBcc:\fP) to display in the pager if header
   ** weeding is turned on. In case the number of lines exeeds its value, the
   ** last line will have 3 dots appended.
   */
  {"max_line_length", DT_NUM, UL &MaxLineLength, "0" },
  /*
   ** .pp
   ** When \fIset\fP, the maximum line length for displaying ``format = flowed'' messages is limited
   ** to this length. A value of 0 (which is also the default) means that the
   ** maximum line length is determined by the terminal width and $$wrapmargin.
   */
  {"maildir_trash", DT_BOOL, OPTMAILDIRTRASH, "no" },
  /*
   ** .pp
   ** If \fIset\fP, messages marked as deleted will be saved with the maildir
   ** (T)rashed flag instead of physically deleted.
   ** .pp
   ** \fBNOTE:\fP this only applies
   ** to maildir-style mailboxes. Setting it will have no effect on other
   ** mailbox types.
   ** .pp
   ** It is similiar to the trash option.
   */
  {"mark_old", DT_BOOL, OPTMARKOLD, "yes" },
  /*
   ** .pp
   ** Controls whether or not Madmutt marks \fInew\fP \fBunread\fP
   ** messages as \fIold\fP if you exit a mailbox without reading them.
   ** .pp
   ** With this option \fIset\fP, the next time you start Madmutt, the messages
   ** will show up with an "O" next to them in the ``index'' menu,
   ** indicating that they are old.
   */
  {"markers", DT_BOOL, OPTMARKERS, "yes" },
  /*
   ** .pp
   ** Controls the display of wrapped lines in the internal pager. If set, a
   ** ``\fT+\fP'' marker is displayed at the beginning of wrapped lines. Also see
   ** the ``$$smart_wrap'' variable.
   */
  {"mask", DT_RX, UL &Mask, "!^\\.[^.]"},
  /*
   ** .pp
   ** A regular expression used in the file browser, optionally preceded by
   ** the \fInot\fP operator ``\fT!\fP''.  Only files whose names match this mask
   ** will be shown. The match is always case-sensitive.
   */
  {"mbox", DT_PATH, UL &Inbox, "~/mbox"},
  /*
   ** .pp
   ** This specifies the folder into which read mail in your ``$$spoolfile''
   ** folder will be appended.
   */
  {"sidebar_boundary", DT_STR, UL &SidebarBoundary, "." },
  /*
   ** .pp
   ** When the sidebar is displayed and $$sidebar_shorten_hierarchy is \fIset\fP, this
   ** variable specifies the characters at which to split a folder name into
   ** ``hierarchy items.''
   */
  {"sidebar_visible", DT_BOOL, OPTMBOXPANE, "no" },
  /*
   ** .pp
   ** This specifies whether or not to show the sidebar (a list of folders specified
   ** with the ``mailboxes'' command).
   */
  {"sidebar_width", DT_NUM, UL &SidebarWidth, "0" },
  /*
   ** .pp
   ** The width of the sidebar.
   */
  {"sidebar_newmail_only", DT_BOOL, OPTSIDEBARNEWMAILONLY, "no" },
  /*
   ** .pp
   ** If \fIset\fP, only folders with new mail will be shown in the sidebar.
   */
  {"sidebar_number_format", DT_STR, UL &SidebarNumberFormat, "%m%?n?(%n)?%?f?[%f]?"},
  /*
   ** .pp
   ** This variable controls how message counts are printed when the sidebar
   ** is enabled. If this variable is \fIempty\fP (\fIand only if\fP), no numbers
   ** will be printed \fIand\fP Madmutt won't frequently count mail (which
   ** may be a great speedup esp. with mbox-style mailboxes.)
   ** .pp
   ** The following \fTprintf(3)\fP-like sequences are supported all of which
   ** may be printed non-zero:
   ** .pp
   ** .dl
   ** .dt %d .dd Number of deleted messages. 1)
   ** .dt %F .dd Number of flagged messages.
   ** .dt %m .dd Total number of messages.
   ** .dt %M .dd Total number of messages shown, i.e. not hidden by a limit. 1)
   ** .dt %n .dd Number of new messages.
   ** .dt %t .dd Number of tagged messages. 1)
   ** .dt %u .dd Number of unread messages.
   ** .de
   ** .pp
   ** 1) These expandos only have a non-zero value for the current mailbox and
   ** will always be zero otherwise.
   */
  {"sidebar_shorten_hierarchy", DT_BOOL, OPTSHORTENHIERARCHY, "no" },
  /*
   ** .pp
   ** When \fIset\fP, the ``hierarchy'' of the sidebar entries will be shortened
   ** only if they cannot be printed in full length (because ``$$sidebar_width''
   ** is set to a too low value). For example, if the newsgroup name
   ** ``de.alt.sysadmin.recovery'' doesn't fit on the screen, it'll get shortened
   ** ``d.a.s.recovery'' while ``de.alt.d0'' still would and thus will not get
   ** shortened.
   ** .pp
   ** At which characters this compression is done is controled via the
   ** $$sidebar_boundary variable.
   */
  {"mbox_type", DT_MAGIC, UL &DefaultMagic, "mbox" },
  /*
   ** .pp
   ** The default mailbox type used when creating new folders. May be any of
   ** \fTmbox\fP, \fTMH\fP and \fTMaildir\fP.
   */
  {"metoo", DT_BOOL, OPTMETOO, "no" },
  /*
   ** .pp
   ** If \fIunset\fP, Madmutt will remove your address (see the ``alternates''
   ** command) from the list of recipients when replying to a message.
   */
  {"menu_context", DT_NUM, UL &MenuContext, "0" },
  /*
   ** .pp
   ** This variable controls the number of lines of context that are given
   ** when scrolling through menus. (Similar to ``$$pager_context''.)
   */
  {"mh_purge", DT_BOOL, OPTMHPURGE, "no" },
  /*
   ** .pp
   ** When \fIunset\fP, Madmutt will mimic mh's behaviour and rename deleted messages
   ** to \fI,<old file name>\fP in mh folders instead of really deleting
   ** them.  If the variable is set, the message files will simply be
   ** deleted.
   */
  {"mh_seq_flagged", DT_STR, UL &MhFlagged, "flagged"},
  /*
   ** .pp
   ** The name of the MH sequence used for flagged messages.
   */
  {"mh_seq_replied", DT_STR, UL &MhReplied, "replied"},
  /*
   ** .pp
   ** The name of the MH sequence used to tag replied messages.
   */
  {"mh_seq_unseen", DT_STR, UL &MhUnseen, "unseen"},
  /*
   ** .pp
   ** The name of the MH sequence used for unseen messages.
   */
  {"mime_forward", DT_QUAD, OPT_MIMEFWD, "no" },
  /*
   ** .pp
   ** When \fIset\fP, the message you are forwarding will be attached as a
   ** separate MIME part instead of included in the main body of the
   ** message.
   ** .pp
   ** This is useful for forwarding MIME messages so the receiver
   ** can properly view the message as it was delivered to you. If you like
   ** to switch between MIME and not MIME from mail to mail, set this
   ** variable to ask-no or ask-yes.
   ** .pp
   ** Also see ``$$forward_decode'' and ``$$mime_forward_decode''.
   */
  {"mime_forward_decode", DT_BOOL, OPTMIMEFORWDECODE, "no" },
  /*
   ** .pp
   ** Controls the decoding of complex MIME messages into \fTtext/plain\fP when
   ** forwarding a message while ``$$mime_forward'' is \fIset\fP. Otherwise
   ** ``$$forward_decode'' is used instead.
   */
  {"mime_forward_rest", DT_QUAD, OPT_MIMEFWDREST, "yes" },
  /*
   ** .pp
   ** When forwarding multiple attachments of a MIME message from the recvattach
   ** menu, attachments which cannot be decoded in a reasonable manner will
   ** be attached to the newly composed message if this option is set.
   */
  {"move", DT_QUAD, OPT_MOVE, "ask-no" },
  /*
   ** .pp
   ** Controls whether or not Madmutt will move read messages
   ** from your spool mailbox to your ``$$mbox'' mailbox, or as a result of
   ** a ``$mbox-hook'' command.
   */
  {"message_format", DT_STR, UL &MsgFmt, "%s"},
  /*
   ** .pp
   ** This is the string displayed in the ``attachment'' menu for
   ** attachments of type \fTmessage/rfc822\fP.  For a full listing of defined
   ** \fTprintf(3)\fP-like sequences see the section on ``$$index_format''.
   */
  {"msgid_format", DT_STR, UL &MsgIdFormat, "%Y%m%d%h%M%s.G%P%p"},
  /*
   ** .pp
   ** This is the format for the ``local part'' of the \fTMessage-ID:\fP header
   ** field generated by Madmutt. If this variable is empty, no \fTMessage-ID:\fP
   ** headers will be generated. The '%'
   ** character marks that certain data will be added to the string, similar to
   ** \fTprintf(3)\fP. The following characters are allowed:
   ** .pp
   ** .dl
   ** .dt %d .dd the current day of month
   ** .dt %h .dd the current hour
   ** .dt %m .dd the current month
   ** .dt %M .dd the current minute
   ** .dt %O .dd the current UNIX timestamp (octal)
   ** .dt %p .dd the process ID
   ** .dt %P .dd the current Message-ID prefix (a character rotating with
   **            every Message-ID being generated)
   ** .dt %r .dd a random integer value (decimal)
   ** .dt %R .dd a random integer value (hexadecimal)
   ** .dt %s .dd the current second
   ** .dt %T .dd the current UNIX timestamp (decimal)
   ** .dt %X .dd the current UNIX timestamp (hexadecimal)
   ** .dt %Y .dd the current year (Y2K compliant)
   ** .dt %% .dd the '%' character
   ** .de
   ** .pp
   ** \fBNote:\fP Please only change this setting if you know what you are doing.
   ** Also make sure to consult RFC2822 to produce technically \fIvalid\fP strings.
   */
  {"narrow_tree", DT_BOOL, OPTNARROWTREE, "no" },
  /*
   ** .pp
   ** This variable, when \fIset\fP, makes the thread tree narrower, allowing
   ** deeper threads to fit on the screen.
   */
  { "net_inc", DT_NUM, UL &NetInc, "10" },
  /*
  ** .pp
  ** Operations that expect to transfer a large amount of data over the
  ** network will update their progress every \fInet_inc\fP kilobytes.
  ** If set to 0, no progress messages will be displayed.
  ** .pp
  ** See also ``$$read_inc'' and ``$$write_inc''.
  */
  {"pager_context", DT_NUM, UL &PagerContext, "0" },
  /*
   ** .pp
   ** This variable controls the number of lines of context that are given
   ** when displaying the next or previous page in the internal pager.  By
   ** default, Madmutt will display the line after the last one on the screen
   ** at the top of the next page (0 lines of context).
   */
  {"pager_format", DT_STR, UL &PagerFmt, "-%Z- %C/%m: %-20.20n   %s"},
  /*
   ** .pp
   ** This variable controls the format of the one-line message ``status''
   ** displayed before each message in either the internal or an external
   ** pager.  The valid sequences are listed in the ``$$index_format''
   ** section.
   */
  {"pager_index_lines", DT_NUM, UL &PagerIndexLines, "0" },
  /*
   ** .pp
   ** Determines the number of lines of a mini-index which is shown when in
   ** the pager.  The current message, unless near the top or bottom of the
   ** folder, will be roughly one third of the way down this mini-index,
   ** giving the reader the context of a few messages before and after the
   ** message.  This is useful, for example, to determine how many messages
   ** remain to be read in the current thread.  One of the lines is reserved
   ** for the status bar from the index, so a \fIpager_index_lines\fP of 6
   ** will only show 5 lines of the actual index.  A value of 0 results in
   ** no index being shown.  If the number of messages in the current folder
   ** is less than \fIpager_index_lines\fP, then the index will only use as
   ** many lines as it needs.
   */
  {"pager_stop", DT_BOOL, OPTPAGERSTOP, "no" },
  /*
   ** .pp
   ** When \fIset\fP, the internal-pager will \fBnot\fP move to the next message
   ** when you are at the end of a message and invoke the \fInext-page\fP
   ** function.
   */
  {"pgp_retainable_sigs", DT_BOOL, OPTPGPRETAINABLESIG, "no" },
  /*
   ** .pp
   ** If \fIset\fP, signed and encrypted messages will consist of nested
   ** multipart/signed and multipart/encrypted body parts.
   ** .pp
   ** This is useful for applications like encrypted and signed mailing
   ** lists, where the outer layer (multipart/encrypted) can be easily
   ** removed, while the inner multipart/signed part is retained.
   ** (PGP only)
   */
  {"pgp_show_unusable", DT_BOOL, OPTPGPSHOWUNUSABLE, "yes" },
  /*
   ** .pp
   ** If \fIset\fP, Madmutt will display non-usable keys on the PGP key selection
   ** menu.  This includes keys which have been revoked, have expired, or
   ** have been marked as ``disabled'' by the user.
   ** (PGP only)
   */
  {"pgp_sign_as", DT_STR, UL &PgpSignAs, "" },
  /*
   ** .pp
   ** If you have more than one key pair, this option allows you to specify
   ** which of your private keys to use.  It is recommended that you use the
   ** keyid form to specify your key (e.g., ``\fT0x00112233\fP'').
   ** (PGP only)
   */
  {"pgp_sort_keys", DT_SORT|DT_SORT_KEYS, UL &PgpSortKeys, "address" },
  /*
   ** .pp
   ** Specifies how the entries in the ``pgp keys'' menu are sorted. The
   ** following are legal values:
   ** .pp
   ** .dl
   ** .dt address .dd sort alphabetically by user id
   ** .dt keyid   .dd sort alphabetically by key id
   ** .dt date    .dd sort by key creation date
   ** .dt trust   .dd sort by the trust of the key
   ** .de
   ** .pp
   ** If you prefer reverse order of the above values, prefix it with
   ** ``reverse-''.
   ** (PGP only)
   */
  {"forward_decrypt", DT_BOOL, OPTFORWDECRYPT, "yes" },
  /*
   ** .pp
   ** Controls the handling of encrypted messages when forwarding a message.
   ** When \fIset\fP, the outer layer of encryption is stripped off.  This
   ** variable is only used if ``$$mime_forward'' is \fIset\fP and
   ** ``$$mime_forward_decode'' is \fIunset\fP.
   ** (PGP only)
   */
  {"smime_encrypt_with", DT_STR, UL &SmimeCryptAlg, "" },
  /*
   ** .pp
   ** This sets the algorithm that should be used for encryption.
   ** Valid choices are ``\fTdes\fP'', ``\fTdes3\fP'', ``\fTrc2-40\fP'',
   ** ``\fTrc2-64\fP'', ``\frc2-128\fP''.
   ** .pp
   ** If \fIunset\fP ``\fI3des\fP'' (TripleDES) is used.
   ** (S/MIME only)
   */
  {"smime_default_key", DT_STR, UL &SmimeDefaultKey, "" },
  /*
   ** .pp
   ** This is the default key-pair to use for signing. This must be set to the
   ** keyid (the hash-value that OpenSSL generates) to work properly
   ** (S/MIME only)
   */
  {"pipe_split", DT_BOOL, OPTPIPESPLIT, "no" },
  /*
   ** .pp
   ** Used in connection with the \fIpipe-message\fP command and the ``tag-
   ** prefix'' or ``tag-prefix-cond'' operators.
   ** If this variable is \fIunset\fP, when piping a list of
   ** tagged messages Madmutt will concatenate the messages and will pipe them
   ** as a single folder.  When \fIset\fP, Madmutt will pipe the messages one by one.
   ** In both cases the messages are piped in the current sorted order,
   ** and the ``$$pipe_sep'' separator is added after each message.
   */
  {"pipe_decode", DT_BOOL, OPTPIPEDECODE, "no" },
  /*
   ** .pp
   ** Used in connection with the \fIpipe-message\fP command.  When \fIunset\fP,
   ** Madmutt will pipe the messages without any preprocessing. When \fIset\fP, Madmutt
   ** will weed headers and will attempt to PGP/MIME decode the messages
   ** first.
   */
  {"pipe_sep", DT_STR, UL &PipeSep, "\n"},
  /*
   ** .pp
   ** The separator to add between messages when piping a list of tagged
   ** messages to an external Unix command.
   */
  {"pop_authenticators", DT_STR, UL &PopAuthenticators, "" },
  /*
   ** .pp
   ** This is a colon-delimited list of authentication methods Madmutt may
   ** attempt to use to log in to an POP server, in the order Madmutt should
   ** try them.  Authentication methods are either ``\fTuser\fP'', ``\fTapop\fP''
   ** or any SASL mechanism, eg ``\fTdigest-md5\fP'', ``\fTgssapi\fP'' or ``\fTcram-md5\fP''.
   ** .pp
   ** This parameter is case-insensitive. If this parameter is \fIunset\fP
   ** (the default) Madmutt will try all available methods, in order from
   ** most-secure to least-secure.
   ** .pp
   ** Example: \fTset pop_authenticators="digest-md5:apop:user"\fP
   */
  {"pop_mail_check", DT_NUM, UL &PopCheckTimeout, "60" },
  /*
   ** .pp
   ** This variable configures how often (in seconds) Madmutt should look for
   ** new mail.
   */
  {"pop_delete", DT_QUAD, OPT_POPDELETE, "ask-no" },
  /*
   ** .pp
   ** If \fIset\fP, Madmutt will delete successfully downloaded messages from the POP
   ** server when using the ``fetch-mail'' function.  When \fIunset\fP, Madmutt will
   ** download messages but also leave them on the POP server.
   */
  {"pop_host", DT_STR, UL &PopHost, ""},
  /*
   ** .pp
   ** The name of your POP server for the ``fetch-mail'' function.  You
   ** can also specify an alternative port, username and password, i.e.:
   ** .pp
   ** \fT[pop[s]://][username[:password]@]popserver[:port]\fP
   ** .pp
   ** \fBNote:\fP Storing passwords in a configuration file
   ** presents a security risk since the superuser of your machine may read it
   ** regardless of the file's permissions.
   */
  {"pop_last", DT_BOOL, OPTPOPLAST, "no" },
  /*
   ** .pp
   ** If this variable is \fIset\fP, Madmutt will try to use the ``\fTLAST\fP'' POP command
   ** for retrieving only unread messages from the POP server when using
   ** the ``fetch-mail'' function.
   */
  {"pop_reconnect", DT_QUAD, OPT_POPRECONNECT, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not Madmutt will try to reconnect to a POP server if the
   ** connection is lost.
   */
  {"pop_user", DT_STR, UL &PopUser, "" },
  /*
   ** .pp
   ** Your login name on the POP server.
   ** .pp
   ** This variable defaults to your user name on the local machine.
   */
  {"pop_pass", DT_STR, UL &PopPass, ""},
  /*
   ** .pp
   ** Specifies the password for your POP account.  If \fIunset\fP, Madmutt will
   ** prompt you for your password when you open POP mailbox.
   ** .pp
   ** \fBNote:\fP Storing passwords in a configuration file
   ** presents a security risk since the superuser of your machine may read it
   ** regardless of the file's permissions.
   */
  {"postpone", DT_QUAD, OPT_POSTPONE, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not messages are saved in the ``$$postponed''
   ** mailbox when you elect not to send immediately.
   */
  {"postponed", DT_PATH, UL &Postponed, "~/postponed"},
  /*
   ** .pp
   ** Madmutt allows you to indefinitely ``$postpone sending a message'' which
   ** you are editing.  When you choose to postpone a message, Madmutt saves it
   ** in the mailbox specified by this variable.  Also see the ``$$postpone''
   ** variable.
   */
  {"preconnect", DT_STR, UL &Preconnect, "" },
  /*
   ** .pp
   ** If \fIset\fP, a shell command to be executed if Madmutt fails to establish
   ** a connection to the server. This is useful for setting up secure
   ** connections, e.g. with \fTssh(1)\fP. If the command returns a  nonzero
   ** status, Madmutt gives up opening the server. Example:
   ** .pp
   ** \fTpreconnect="ssh -f -q -L 1234:mailhost.net:143 mailhost.net
   **                sleep 20 < /dev/null > /dev/null"\fP
   ** .pp
   ** Mailbox ``foo'' on mailhost.net can now be reached
   ** as ``{localhost:1234}foo''.
   ** .pp
   ** \fBNote:\fP For this example to work, you must be able to log in to the
   ** remote machine without having to enter a password.
   */
  {"print", DT_QUAD, OPT_PRINT, "ask-no" },
  /*
   ** .pp
   ** Controls whether or not Madmutt really prints messages.
   ** This is set to \fIask-no\fP by default, because some people
   ** accidentally hit ``p'' often.
   */
  {"print_command", DT_PATH, UL &PrintCmd, "lpr"},
  /*
   ** .pp
   ** This specifies the command pipe that should be used to print messages.
   */
  {"print_decode", DT_BOOL, OPTPRINTDECODE, "yes" },
  /*
   ** .pp
   ** Used in connection with the print-message command.  If this
   ** option is \fIset\fP, the message is decoded before it is passed to the
   ** external command specified by $$print_command.  If this option
   ** is \fIunset\fP, no processing will be applied to the message when
   ** printing it.  The latter setting may be useful if you are using
   ** some advanced printer filter which is able to properly format
   ** e-mail messages for printing.
   */
  {"print_split", DT_BOOL, OPTPRINTSPLIT, "no" },
  /*
   ** .pp
   ** Used in connection with the print-message command.  If this option
   ** is \fIset\fP, the command specified by $$print_command is executed once for
   ** each message which is to be printed.  If this option is \fIunset\fP,
   ** the command specified by $$print_command is executed only once, and
   ** all the messages are concatenated, with a form feed as the message
   ** separator.
   ** .pp
   ** Those who use the \fTenscript(1)\fP program's mail-printing mode will
   ** most likely want to set this option.
   */
  {"prompt_after", DT_BOOL, OPTPROMPTAFTER, "yes" },
  /*
   ** .pp
   ** If you use an \fIexternal\fP ``$$pager'', setting this variable will
   ** cause Madmutt to prompt you for a command when the pager exits rather
   ** than returning to the index menu.  If \fIunset\fP, Madmutt will return to the
   ** index menu when the external pager exits.
   */
  {"query_command", DT_PATH, UL &QueryCmd, ""},
  /*
   ** .pp
   ** This specifies the command that Madmutt will use to make external address
   ** queries.  The string should contain a \fT%s\fP, which will be substituted
   ** with the query string the user types.  See ``$query'' for more
   ** information.
   */
  {"quote_empty", DT_BOOL, OPTQUOTEEMPTY, "yes" },
  /*
   ** .pp
   ** Controls whether or not empty lines will be quoted using
   ** ``$indent_string''.
   */
  {"quote_quoted", DT_BOOL, OPTQUOTEQUOTED, "no" },
  /*
   ** .pp
   ** Controls how quoted lines will be quoted. If \fIset\fP, one quote
   ** character will be added to the end of existing prefix.  Otherwise,
   ** quoted lines will be prepended by ``$indent_string''.
   */
  {"quote_regexp", DT_RX, UL &QuoteRegexp, "^([ \t]*[|>:}#])+"},
  /*
   ** .pp
   ** A regular expression used in the internal-pager to determine quoted
   ** sections of text in the body of a message.
   ** .pp
   ** \fBNote:\fP In order to use the \fIquoted\fP\fBx\fP patterns in the
   ** internal pager, you need to set this to a regular expression that
   ** matches \fIexactly\fP the quote characters at the beginning of quoted
   ** lines.
   */
  {"read_inc", DT_NUM, UL &ReadInc, "10" },
  /*
   ** .pp
   ** If set to a value greater than 0, Madmutt will display which message it
   ** is currently on when reading a mailbox.  The message is printed after
   ** \fIread_inc\fP messages have been read (e.g., if set to 25, Madmutt will
   ** print a message when it reads message 25, and then again when it gets
   ** to message 50).  This variable is meant to indicate progress when
   ** reading large mailboxes which may take some time.
   ** When set to 0, only a single message will appear before the reading
   ** the mailbox.
   ** .pp
   ** Also see the ``$$write_inc'' variable.
   */
  {"realname", DT_STR, UL &Realname, "" },
  /*
   ** .pp
   ** This variable specifies what ``real'' or ``personal'' name should be used
   ** when sending messages.
   ** .pp
   ** By default, this is the GECOS field from \fT/etc/passwd\fP.
   ** .pp
   ** \fINote:\fP This
   ** variable will \fInot\fP be used when the user has set a real name
   ** in the $$from variable.
   */
  {"recall", DT_QUAD, OPT_RECALL, "ask-yes" },
  /*
   ** .pp
   ** Controls whether or not Madmutt recalls postponed messages
   ** when composing a new message.  Also see ``$$postponed''.
   ** .pp
   ** Setting this variable to \fIyes\fP is not generally useful, and thus not
   ** recommended.
   */
  {"reply_regexp", DT_RX, UL &ReplyRegexp, "^(re([\\[0-9\\]+])*|aw):[ \t]*"},
  /*
   ** .pp
   ** A regular expression used to recognize reply messages when threading
   ** and replying. The default value corresponds to the English ``Re:'' and
   ** the German ``Aw:''.
   */
  {"reply_to", DT_QUAD, OPT_REPLYTO, "ask-yes" },
  /*
   ** .pp
   ** If \fIset\fP, when replying to a message, Madmutt will use the address listed
   ** in the ``\fTReply-To:\fP'' header field as the recipient of the reply.  If \fIunset\fP,
   ** it will use the address in the ``\fTFrom:\fP'' header field instead.
   ** .pp
   ** This
   ** option is useful for reading a mailing list that sets the ``\fTReply-To:\fP''
   ** header field to the list address and you want to send a private
   ** message to the author of a message.
   */
  {"resolve", DT_BOOL, OPTRESOLVE, "yes" },
  /*
   ** .pp
   ** When set, the cursor will be automatically advanced to the next
   ** (possibly undeleted) message whenever a command that modifies the
   ** current message is executed.
   */
  {"reverse_alias", DT_BOOL, OPTREVALIAS, "no" },
  /*
   ** .pp
   ** This variable controls whether or not Madmutt will display the ``personal''
   ** name from your aliases in the index menu if it finds an alias that
   ** matches the message's sender.  For example, if you have the following
   ** alias:
   ** .pp
   **  \fTalias juser abd30425@somewhere.net (Joe User)\fP
   ** .pp
   ** and then you receive mail which contains the following header:
   ** .pp
   **  \fTFrom: abd30425@somewhere.net\fP
   ** .pp
   ** It would be displayed in the index menu as ``Joe User'' instead of
   ** ``abd30425@somewhere.net.''  This is useful when the person's e-mail
   ** address is not human friendly (like CompuServe addresses).
   */
  {"reverse_name", DT_BOOL, OPTREVNAME, "no" },
  /*
   ** .pp
   ** It may sometimes arrive that you receive mail to a certain machine,
   ** move the messages to another machine, and reply to some the messages
   ** from there.  If this variable is \fIset\fP, the default \fTFrom:\fP line of
   ** the reply messages is built using the address where you received the
   ** messages you are replying to \fBif\fP that address matches your
   ** alternates.  If the variable is \fIunset\fP, or the address that would be
   ** used doesn't match your alternates, the \fTFrom:\fP line will use
   ** your address on the current machine.
   */
  {"reverse_realname", DT_BOOL, OPTREVREAL, "yes" },
  /*
   ** .pp
   ** This variable fine-tunes the behaviour of the $reverse_name feature.
   ** When it is \fIset\fP, Madmutt will use the address from incoming messages as-is,
   ** possibly including eventual real names.  When it is \fIunset\fP, Madmutt will
   ** override any such real names with the setting of the $realname variable.
   */
  {"sig_dashes", DT_BOOL, OPTSIGDASHES, "yes" },
  /*
   ** .pp
   ** If set, a line containing ``\fT-- \fP'' (dash, dash, space)
   ** will be inserted before your ``$$signature''.  It is \fBstrongly\fP
   ** recommended that you not unset this variable unless your ``signature''
   ** contains just your name. The reason for this is because many software
   ** packages use ``\fT-- \n\fP'' to detect your signature.
   ** .pp
   ** For example, Madmutt has the ability to highlight
   ** the signature in a different color in the builtin pager.
   */
  {"signoff_string", DT_STR, UL &SignOffString, "" },
  /*
   ** .pp
   ** If \fIset\fP, this string will be inserted before the signature. This is useful
   ** for people that want to sign off every message they send with their name.
   ** .pp
   ** If you want to insert your website's URL, additional contact information or
   ** witty quotes into your mails, better use a signature file instead of
   ** the signoff string.
   */
  {"simple_search", DT_STR, UL &SimpleSearch, "~f %s | ~s %s"},
  /*
   ** .pp
   ** Specifies how Madmutt should expand a simple search into a real search
   ** pattern.  A simple search is one that does not contain any of the ~
   ** operators.  See ``$patterns'' for more information on search patterns.
   ** .pp
   ** For example, if you simply type ``joe'' at a search or limit prompt, Madmutt
   ** will automatically expand it to the value specified by this variable.
   ** For the default value it would be:
   ** .pp
   ** \fT~f joe | ~s joe\fP
   */
  {"smart_wrap", DT_BOOL, OPTWRAP, "yes" },
  /*
   ** .pp
   ** Controls the display of lines longer than the screen width in the
   ** internal pager. If \fIset\fP, long lines are wrapped at a word boundary.
   ** If \fIunset\fP, lines are simply wrapped at the screen edge. Also see the
   ** ``$$markers'' variable.
   */
  {"smileys", DT_RX, UL &Smileys, "(>From )|(:[-^]?[][)(><}{|/DP])"},
  /*
   ** .pp
   ** The \fIpager\fP uses this variable to catch some common false
   ** positives of ``$$quote_regexp'', most notably smileys in the beginning
   ** of a line
   */
  {"sleep_time", DT_NUM, UL &SleepTime, "1" },
  /*
   ** .pp
   ** Specifies time, in seconds, to pause while displaying certain informational
   ** messages, while moving from folder to folder and after expunging
   ** messages from the current folder.  The default is to pause one second, so
   ** a value of zero for this option suppresses the pause.
   */
  {"sort", DT_SORT, UL &Sort, "date" },
  /*
   ** .pp
   ** Specifies how to sort messages in the \fIindex\fP menu.  Valid values
   ** are:
   ** .pp
   ** .ts
   ** .  date or date-sent
   ** .  date-received
   ** .  from
   ** .  mailbox-order (unsorted)
   ** .  score
   ** .  size
   ** .  spam
   ** .  subject
   ** .  threads
   ** .  to
   ** .te
   ** .pp
   ** You may optionally use the ``reverse-'' prefix to specify reverse sorting
   ** order (example: \fTset sort=reverse-date-sent\fP).
   */
  {"sort_alias", DT_SORT|DT_SORT_ALIAS, UL &SortAlias, "alias" },
  /*
   ** .pp
   ** Specifies how the entries in the ``alias'' menu are sorted.  The
   ** following are legal values:
   ** .pp
   ** .ts
   ** .  address (sort alphabetically by email address)
   ** .  alias (sort alphabetically by alias name)
   ** .  unsorted (leave in order specified in .muttrc)
   ** .te
   */
  {"sort_aux", DT_SORT|DT_SORT_AUX, UL &SortAux, "date" },
  /*
   ** .pp
   ** When sorting by threads, this variable controls how threads are sorted
   ** in relation to other threads, and how the branches of the thread trees
   ** are sorted.  This can be set to any value that ``$$sort'' can, except
   ** threads (in that case, Madmutt will just use date-sent).  You can also
   ** specify the ``last-'' prefix in addition to ``reverse-'' prefix, but last-
   ** must come after reverse-.  The last- prefix causes messages to be
   ** sorted against its siblings by which has the last descendant, using
   ** the rest of sort_aux as an ordering.
   ** .pp
   ** For instance, \fTset sort_aux=last-date-received\fP would mean that if
   ** a new message is received in a thread, that thread becomes the last one
   ** displayed (or the first, if you have \fTset sort=reverse-threads\fP.)
   ** .pp
   ** \fBNote:\fP For reversed ``$$sort'' order $$sort_aux is reversed again
   ** (which is not the right thing to do, but kept to not break any existing
   ** configuration setting).
   */
  {"sort_browser", DT_SORT|DT_SORT_BROWSER, UL &BrowserSort, "alpha" },
  /*
   ** .pp
   ** Specifies how to sort entries in the file browser.  By default, the
   ** entries are sorted alphabetically.  Valid values:
   ** .pp
   ** .ts
   ** .  alpha (alphabetically)
   ** .  date
   ** .  size
   ** .  unsorted
   ** .te
   ** .pp
   ** You may optionally use the ``reverse-'' prefix to specify reverse sorting
   ** order (example: \fTset sort_browser=reverse-date\fP).
   */
  {"sort_re", DT_BOOL, OPTSORTRE, "yes" },
  /*
   ** .pp
   ** This variable is only useful when sorting by threads with
   ** ``$$strict_threads'' \fIunset\fP. In that case, it changes the heuristic
   ** Madmutt uses to thread messages by subject.  With $$sort_re \fIset\fP,
   ** Madmutt will only attach a message as the child of another message by
   ** subject if the subject of the child message starts with a substring
   ** matching the setting of ``$$reply_regexp''. With $$sort_re \fIunset\fP,
   ** Madmutt will attach the message whether or not this is the case,
   ** as long as the non-``$$reply_regexp'' parts of both messages are identical.
   */
  {"spoolfile", DT_PATH, UL &Spoolfile, "" },
  /*
   ** .pp
   ** If your spool mailbox is in a non-default place where Madmutt cannot find
   ** it, you can specify its location with this variable.  Madmutt will
   ** automatically set this variable to the value of the environment
   ** variable $$$MAIL if it is not set.
   */
  {"status_chars", DT_STR, UL &StChars, "-*%A"},
  /*
   ** .pp
   ** Controls the characters used by the ``\fT%r\fP'' indicator in
   ** ``$$status_format''. The first character is used when the mailbox is
   ** unchanged. The second is used when the mailbox has been changed, and
   ** it needs to be resynchronized. The third is used if the mailbox is in
   ** read-only mode, or if the mailbox will not be written when exiting
   ** that mailbox (You can toggle whether to write changes to a mailbox
   ** with the toggle-write operation, bound by default to ``\fT%\fP'').
   ** The fourth  is used to indicate that the current folder has been
   ** opened in attach-message mode (Certain operations like composing
   ** a new mail, replying, forwarding, etc. are not permitted in this mode).
   */
  {"status_format", DT_STR, UL &Status, "-%r-Madmutt: %f [Msgs:%?M?%M/?%m%?n? New:%n?%?o? Old:%o?%?d? Del:%d?%?F? Flag:%F?%?t? Tag:%t?%?p? Post:%p?%?b? Inc:%b?%?l? %l?]---(%s/%S)-%>-(%P)---"},
  /*
   ** .pp
   ** Controls the format of the status line displayed in the \fIindex\fP
   ** menu.  This string is similar to ``$$index_format'', but has its own
   ** set of \fTprintf(3)\fP-like sequences:
   ** .pp
   ** .dl
   ** .dt %b  .dd number of mailboxes with new mail *
   ** .dt %B  .dd the short pathname of the current mailbox
   ** .dt %d  .dd number of deleted messages *
   ** .dt %f  .dd the full pathname of the current mailbox
   ** .dt %F  .dd number of flagged messages *
   ** .dt %h  .dd local hostname
   ** .dt %l  .dd size (in bytes) of the current mailbox *
   ** .dt %L  .dd size (in bytes) of the messages shown
   **             (i.e., which match the current limit) *
   ** .dt %m  .dd the number of messages in the mailbox *
   ** .dt %M  .dd the number of messages shown (i.e., which match the current limit) *
   ** .dt %n  .dd number of new messages in the mailbox *
   ** .dt %o  .dd number of old unread messages *
   ** .dt %p  .dd number of postponed messages *
   ** .dt %P  .dd percentage of the way through the index
   ** .dt %r  .dd modified/read-only/won't-write/attach-message indicator,
   **             according to $$status_chars
   ** .dt %s  .dd current sorting mode ($$sort)
   ** .dt %S  .dd current aux sorting method ($$sort_aux)
   ** .dt %t  .dd number of tagged messages *
   ** .dt %u  .dd number of unread messages *
   ** .dt %v  .dd Madmutt version string
   ** .dt %V  .dd currently active limit pattern, if any *
   ** .dt %>X .dd right justify the rest of the string and pad with "X"
   ** .dt %|X .dd pad to the end of the line with "X"
   ** .de
   ** .pp
   ** * = can be optionally printed if nonzero
   ** .pp
   ** Some of the above sequences can be used to optionally print a string
   ** if their value is nonzero.  For example, you may only want to see the
   ** number of flagged messages if such messages exist, since zero is not
   ** particularly meaningful.  To optionally print a string based upon one
   ** of the above sequences, the following construct is used
   ** .pp
   **  \fT%?<sequence_char>?<optional_string>?\fP
   ** .pp
   ** where \fIsequence_char\fP is a character from the table above, and
   ** \fIoptional_string\fP is the string you would like printed if
   ** \fIsequence_char\fP is nonzero.  \fIoptional_string\fP \fBmay\fP contain
   ** other sequences as well as normal text, but you may \fBnot\fP nest
   ** optional strings.
   ** .pp
   ** Here is an example illustrating how to optionally print the number of
   ** new messages in a mailbox:
   ** .pp
   **  \fT%?n?%n new messages.?\fP
   ** .pp
   ** Additionally you can switch between two strings, the first one, if a
   ** value is zero, the second one, if the value is nonzero, by using the
   ** following construct:
   ** .pp
   **  \fT%?<sequence_char>?<if_string>&<else_string>?\fP
   ** .pp
   ** You can additionally force the result of any \fTprintf(3)\fP-like sequence
   ** to be lowercase by prefixing the sequence character with an underscore
   ** (\fT_\fP) sign.  For example, if you want to display the local hostname in
   ** lowercase, you would use:
   ** .pp
   **  \fT%_h\fP
   ** .pp
   ** If you prefix the sequence character with a colon (\fT:\fP) character, Madmutt
   ** will replace any dots in the expansion by underscores. This might be helpful
   ** with IMAP folders that don't like dots in folder names.
   */
  {"strict_mailto", DT_BOOL, OPTSTRICTMAILTO, "yes" },
  /*
   **
   ** .pp
   ** With mailto: style links, a body as well as arbitrary header information
   ** may be embedded. This may lead to (user) headers being overwriten without note
   ** if ``$$edit_headers'' is unset.
   **
   ** .pp
   ** If this variable is \fIset\fP, Madmutt is strict and allows anything to be
   ** changed. If it's \fIunset\fP, all headers given will be prefixed with
   ** ``X-Mailto-'' and the message including headers will be shown in the editor
   ** regardless of what ``$$edit_headers'' is set to.
   **/
  {"strict_threads", DT_BOOL, OPTSTRICTTHREADS, "no" },
  /*
   ** .pp
   ** If \fIset\fP, threading will only make use of the ``\fTIn-Reply-To:\fP'' and
   ** ``\fTReferences:\fP'' header fields when you ``$$sort'' by message threads.  By
   ** default, messages with the same subject are grouped together in
   ** ``pseudo threads.''  This may not always be desirable, such as in a
   ** personal mailbox where you might have several unrelated messages with
   ** the subject ``hi'' which will get grouped together. See also
   ** ``$$sort_re'' for a less drastic way of controlling this
   ** behaviour.
   */
  {"strip_was", DT_BOOL, OPTSTRIPWAS, "no" },
  /**
  ** .pp
  ** When \fIset\fP, Madmutt will remove the trailing part of the ``\fTSubject:\fP''
  ** line which matches $$strip_was_regex when replying. This is useful to
  ** properly react on subject changes and reduce ``subject noise.'' (esp. in Usenet)
  **/
  {"strip_was_regex", DT_RX, UL &StripWasRegexp, "\\([Ww][Aa][RrSs]: .*\\)[ ]*$"},
  /**
  ** .pp
  ** When non-empty and $$strip_was is \fIset\fP, Madmutt will remove this
  ** trailing part of the ``Subject'' line when replying if it won't be empty
  ** afterwards.
  **/
  {"stuff_quoted", DT_BOOL, OPTSTUFFQUOTED, "no" },
  /*
   ** .pp
   ** If \fIset\fP, attachments with flowed format will have their quoting ``stuffed'',
   ** i.e. a space will be inserted between the quote characters and the actual
   ** text.
   */
  {"text_flowed", DT_BOOL, OPTTEXTFLOWED, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will generate \fTtext/plain; format=flowed\fP attachments.
   ** This format is easier to handle for some mailing software, and generally
   ** just looks like ordinary text.  To actually make use of this format's
   ** features, you'll need support in your editor.
   ** .pp
   ** Note that $$indent_string is ignored when this option is set.
   */
  {"thread_received", DT_BOOL, OPTTHREADRECEIVED, "no" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt uses the date received rather than the date sent
   ** to thread messages by subject.
   */
  {"thorough_search", DT_BOOL, OPTTHOROUGHSRC, "no" },
  /*
   ** .pp
   ** Affects the \fT~b\fP and \fT~h\fP search operations described in
   ** section ``$patterns'' above.  If \fIset\fP, the headers and attachments of
   ** messages to be searched are decoded before searching.  If \fIunset\fP,
   ** messages are searched as they appear in the folder.
   */
  {"tilde", DT_BOOL, OPTTILDE, "no" },
  /*
   ** .pp
   ** When \fIset\fP, the internal-pager will pad blank lines to the bottom of the
   ** screen with a tilde (~).
   */
  {"timeout", DT_NUM, UL &Timeout, "600" },
  /*
   ** .pp
   ** This variable controls the \fInumber of seconds\fP Madmutt will wait
   ** for a key to be pressed in the main menu before timing out and
   ** checking for new mail.  A value of zero or less will cause Madmutt
   ** to never time out.
   */
  {"to_chars", DT_STR, UL &Tochars, " +TCFL"},
  /*
   ** .pp
   ** Controls the character used to indicate mail addressed to you.  The
   ** first character is the one used when the mail is NOT addressed to your
   ** address (default: space).  The second is used when you are the only
   ** recipient of the message (default: +).  The third is when your address
   ** appears in the ``\fTTo:\fP'' header field, but you are not the only recipient of
   ** the message (default: T).  The fourth character is used when your
   ** address is specified in the ``\fTCc:\fP'' header field, but you are not the only
   ** recipient.  The fifth character is used to indicate mail that was sent
   ** by \fIyou\fP.  The sixth character is used to indicate when a mail
   ** was sent to a mailing-list you're subscribe to (default: L).
   */
  {"trash", DT_PATH, UL &TrashPath, "" },
  /*
   ** .pp
   ** If \fIset\fP, this variable specifies the path of the trash folder where the
   ** mails marked for deletion will be moved, instead of being irremediably
   ** purged.
   ** .pp
   ** \fBNote\fP: When you delete a message in the trash folder, it is really
   ** deleted, so that there is no way to recover mail.
   */
  {"tunnel", DT_STR, UL &Tunnel, "" },
  /*
   ** .pp
   ** Setting this variable will cause Madmutt to open a pipe to a command
   ** instead of a raw socket. You may be able to use this to set up
   ** preauthenticated connections to your IMAP/POP3 server. Example:
   ** .pp
   **  \fTtunnel="ssh -q mailhost.net /usr/local/libexec/imapd"\fP
   ** .pp
   ** \fBNote:\fP For this example to work you must be able to log in to the remote
   ** machine without having to enter a password.
   */
  {"umask", DT_NUM, UL &Umask, "0077" },
  /*
   ** .pp
   ** This sets the umask that will be used by Madmutt when creating all
   ** kinds of files. If \fIunset\fP, the default value is \fT077\fP.
   */
  {"use_from", DT_BOOL, OPTUSEFROM, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will generate the ``\fTFrom:\fP'' header field when
   ** sending messages. If \fIunset\fP, no ``\fTFrom:\fP'' header field will be
   ** generated unless the user explicitly sets one using the ``$my_hdr''
   ** command.
   */
#ifdef HAVE_LIBIDN
  {"use_idn", DT_BOOL, OPTUSEIDN, "yes" },
  /*
   ** .pp
   ** Availability: IDN
   **
   ** .pp
   ** When \fIset\fP, Madmutt will show you international domain names decoded.
   ** .pp
   ** \fBNote:\fP You can use IDNs for addresses even if this is \fIunset\fP.
   ** This variable only affects decoding.
   */
#endif /* HAVE_LIBIDN */
  {"agent_string", DT_BOOL, OPTXMAILER, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will add a ``\fTUser-Agent:\fP'' header to outgoing
   ** messages, indicating which version of Madmutt was used for composing
   ** them.
   */
  {"wait_key", DT_BOOL, OPTWAITKEY, "yes" },
  /*
   ** .pp
   ** Controls whether Madmutt will ask you to press a key after \fIshell-
   ** escape\fP, \fIpipe-message\fP, \fIpipe-entry\fP, \fIprint-message\fP,
   ** and \fIprint-entry\fP commands.
   ** .pp
   ** It is also used when viewing attachments with ``$auto_view'', provided
   ** that the corresponding mailcap entry has a \fTneedsterminal\fP flag,
   ** and the external program is interactive.
   ** .pp
   ** When \fIset\fP, Madmutt will always ask for a key. When \fIunset\fP, Madmutt will wait
   ** for a key only if the external command returned a non-zero status.
   */
  {"weed", DT_BOOL, OPTWEED, "yes" },
  /*
   ** .pp
   ** When \fIset\fP, Madmutt will weed headers when displaying, forwarding,
   ** printing, or replying to messages.
   */
  {"wrap_search", DT_BOOL, OPTWRAPSEARCH, "yes" },
  /*
   ** .pp
   ** Controls whether searches wrap around the end of the mailbox.
   ** .pp
   ** When \fIset\fP, searches will wrap around the first (or last) message. When
   ** \fIunset\fP, searches will not wrap.
   */
  {"wrapmargin", DT_NUM, UL &WrapMargin, "0" },
  /*
   ** .pp
   ** Controls the size of the margin remaining at the right side of
   ** the terminal when Madmutt's pager does smart wrapping.
   */
  {"write_inc", DT_NUM, UL &WriteInc, "10" },
  /*
   ** .pp
   ** When writing a mailbox, a message will be printed every
   ** \fIwrite_inc\fP messages to indicate progress.  If set to 0, only a
   ** single message will be displayed before writing a mailbox.
   ** .pp
   ** Also see the ``$$read_inc'' variable.
   */
  {"write_bcc", DT_BOOL, OPTWRITEBCC, "yes" },
  /*
   ** .pp
   ** Controls whether Madmutt writes out the Bcc header when preparing
   ** messages to be sent.  Exim users may wish to \fIunset\fP this.
   */
  {"xterm_icon", DT_STR, UL &XtermIcon, "M%?n?AIL&ail?"},
  /*
   ** .pp
   ** Controls the format of the X11 icon title, as long as $$xterm_set_titles
   ** is \fIset\fP. This string is identical in formatting to the one used by
   ** ``$$status_format''.
   */
  {"xterm_set_titles", DT_BOOL, OPTXTERMSETTITLES, "no" },
  /*
   ** .pp
   ** Controls whether Madmutt sets the xterm title bar and icon name
   ** (as long as you're in an appropriate terminal). The default must
   ** be \fIunset\fP to force in the validity checking.
   */
  {"xterm_leave", DT_STR, UL &XtermLeave, "" },
  /*
   ** .pp
   ** If $$xterm_set_titles is \fIset\fP, this string will be used to
   ** set the title when leaving Madmutt. For terminal-based programs,
   ** there's no easy and portable way to read the current title so Madmutt
   ** cannot read it upon startup and restore it when exiting.
   **
   ** .pp
   ** Based on the xterm FAQ, the following might work:
   **
   ** .pp
   ** \fTset xterm_leave = "`test x$$$DISPLAY != x && xprop -id $$$WINDOWID | grep WM_NAME | cut -d '"' -f 2`"\fP
   */
  {"xterm_title", DT_STR, UL &XtermTitle, "Madmutt with %?m?%m messages&no messages?%?n? [%n New]?"},
  /*
   ** .pp
   ** Controls the format of the title bar of the xterm provided that
   ** $$xterm_set_titles has been \fIset\fP. This string is identical in formatting
   ** to the one used by ``$$status_format''.
   */
#ifdef USE_NNTP
  {"nntp_post_moderated", DT_QUAD, OPT_TOMODERATED, "ask-yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If set to \fIyes\fP, Madmutt will post articles to newsgroup that have
   ** not permissions to post (e.g. moderated).
   ** .pp
   ** \fBNote:\fP if the newsserver
   ** does not support posting to that newsgroup or a group is totally read-only, that
   ** posting will not have any effect.
   */
  {"nntp_host", DT_STR, UL &NewsServer, "" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** This variable specifies the name (or address) of the NNTP server to be used.
   ** .pp
   ** It
   ** defaults to the value specified via the environment variable
   ** \fT$$$NNTPSERVER\fP or contained in the file \fT/etc/nntpserver\fP.
   ** .pp
   ** You can also
   ** specify a username and an alternative port for each newsserver, e.g.
   ** .pp
   ** \fT[nntp[s]://][username[:password]@]newsserver[:port]\fP
   ** .pp
   ** \fBNote:\fP Using a password as shown and stored in a configuration file
   ** presents a security risk since the superuser of your machine may read it
   ** regardless of the file's permissions.
   */
  {"nntp_context", DT_NUM, UL &NntpContext, "1000" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** This variable controls how many news articles to cache per newsgroup
   ** (if caching is enabled, see $$nntp_cache_dir) and how many news articles
   ** to show in the ``index'' menu.
   ** .pp
   ** If there're more articles than defined with $$nntp_context, all older ones
   ** will be removed/not shown in the index.
   */
  {"nntp_load_description", DT_BOOL, OPTLOADDESC, "yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** This variable controls whether or not descriptions for newsgroups
   ** are to be loaded when subscribing to a newsgroup.
   */
  {"nntp_user", DT_STR, UL &NntpUser, ""},
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** Your login name on the NNTP server.  If \fIunset\fP and the server requires
   ** authentification, Madmutt will prompt you for your account name.
   */
  {"nntp_pass", DT_STR, UL &NntpPass, ""},
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** Your password for NNTP account.
   ** .pp
   ** \fBNote:\fP Storing passwords in a configuration file
   ** presents a security risk since the superuser of your machine may read it
   ** regardless of the file's permissions.
   */
  {"nntp_mail_check", DT_NUM, UL &NewsPollTimeout, "60" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** The time in seconds until any operations on a newsgroup except posting a new
   ** article will cause a recheck for new news. If set to 0, Madmutt will
   ** recheck on each operation in index (stepping, read article, etc.).
   */
  {"nntp_reconnect", DT_QUAD, OPT_NNTPRECONNECT, "ask-yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** Controls whether or not Madmutt will try to reconnect to a newsserver when the
   ** was connection lost.
   */
  {"nntp_group_index_format", DT_STR, UL &GroupFormat, "%4C %M%N %5s  %-45.45f %d"},
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** This variable allows you to customize the newsgroup browser display to
   ** your personal taste.  This string is similar to ``$index_format'', but
   ** has its own set of \fTprintf(3)\fP-like sequences:
   ** .pp
   ** .ts
   ** %C      current newsgroup number
   ** %d      description of newsgroup (retrieved from server)
   ** %f      newsgroup name
   ** %M      ``-'' if newsgroup not allowed for direct post (moderated for example)
   ** %N      ``N'' if newsgroup is new, ``u'' if unsubscribed, blank otherwise
   ** %n      number of new articles in newsgroup
   ** %s      number of unread articles in newsgroup
   ** %>X     right justify the rest of the string and pad with character "X"
   ** %|X     pad to the end of the line with character "X"
   ** .te
   */
  {"nntp_followup_to_poster", DT_QUAD, OPT_FOLLOWUPTOPOSTER, "ask-yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If this variable is \fIset\fP and the keyword "\fTposter\fP" is present in
   ** the \fTFollowup-To:\fP header field, a follow-up to the newsgroup is not
   ** permitted.  The message will be mailed to the submitter of the
   ** message via mail.
   */
  {"nntp_catchup", DT_QUAD, OPT_CATCHUP, "ask-yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If this variable is \fIset\fP, Madmutt will mark all articles in a newsgroup
   ** as read when you leaving it.
   */
  {"nntp_ask_followup_to", DT_BOOL, OPTASKFOLLOWUP, "no" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If \fIset\fP, Madmutt will prompt you for the \fTFollowup-To:\fP header
   ** field before editing the body of an outgoing news article.
   */
  {"nntp_show_new_news", DT_BOOL, OPTSHOWNEWNEWS, "yes" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If \fIset\fP, the newsserver will be asked for new newsgroups on entering
   ** the browser.  Otherwise, it will be done only once for a newsserver.
   ** Also controls whether or not the number of new articles of subscribed
   ** newsgroups will be checked.
   */
  {"nntp_show_only_unread", DT_BOOL, OPTSHOWONLYUNREAD, "no" },
  /*
   ** .pp
   ** Availability: NNTP
   **
   ** .pp
   ** If \fIset\fP, only subscribed newsgroups that contain unread articles
   ** will be displayed in the newsgroup browser.
   */
#endif
  /*--*/
  { NULL, -1, -1, NULL }
};

const struct mapping_t SortMethods[] = {
    {"date", SORT_DATE},
    {"date-sent", SORT_DATE},
    {"date-received", SORT_RECEIVED},
    {"mailbox-order", SORT_ORDER},
    {"subject", SORT_SUBJECT},
    {"from", SORT_FROM},
    {"size", SORT_SIZE},
    {"threads", SORT_THREADS},
    {"to", SORT_TO},
    {"score", SORT_SCORE},
    {"spam", SORT_SPAM},
    {NULL, 0}
};

/* same as SortMethods, but with "threads" replaced by "date" */

const struct mapping_t SortAuxMethods[] = {
    {"date", SORT_DATE},
    {"date-sent", SORT_DATE},
    {"date-received", SORT_RECEIVED},
    {"mailbox-order", SORT_ORDER},
    {"subject", SORT_SUBJECT},
    {"from", SORT_FROM},
    {"size", SORT_SIZE},
    {"threads", SORT_DATE},       /* note: sort_aux == threads
                                   * isn't possible.
                                   */
    {"to", SORT_TO},
    {"score", SORT_SCORE},
    {"spam", SORT_SPAM},
    {NULL, 0}
};


const struct mapping_t SortBrowserMethods[] = {
    {"alpha", SORT_SUBJECT},
    {"date", SORT_DATE},
    {"size", SORT_SIZE},
    {"unsorted", SORT_ORDER},
    {NULL, 0}
};

const struct mapping_t SortAliasMethods[] = {
    {"alias", SORT_ALIAS},
    {"address", SORT_ADDRESS},
    {"unsorted", SORT_ORDER},
    {NULL, 0}
};

const struct mapping_t SortKeyMethods[] = {
    {"address", SORT_ADDRESS},
    {"date", SORT_DATE},
    {"keyid", SORT_KEYID},
    {"trust", SORT_TRUST},
    {NULL, 0}
};


/* functions used to parse commands in a rc file */

static int parse_attachments (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_unattachments (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_alias (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_unalias (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_source (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_set (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_my_hdr (BUFFER *, BUFFER *, unsigned long, BUFFER *);
static int parse_unmy_hdr (BUFFER *, BUFFER *, unsigned long, BUFFER *);

struct command_t {
    const char *name;
    int (*func) (BUFFER *, BUFFER *, unsigned long, BUFFER *);
    unsigned long data;
};

struct command_t Commands[] = {
    {"bind",                mutt_parse_bind,       0},
    {"color",               mutt_parse_color,      0},
    {"exec",                mutt_parse_exec,       0},
    {"append-hook",         mutt_parse_hook,       M_APPENDHOOK},
    {"close-hook",          mutt_parse_hook,       M_CLOSEHOOK},
    {"crypt-hook",          mutt_parse_hook,       M_CRYPTHOOK},
    {"fcc-hook",            mutt_parse_hook,       M_FCCHOOK},
    {"fcc-save-hook",       mutt_parse_hook,       M_FCCHOOK|M_SAVEHOOK},
    {"folder-hook",         mutt_parse_hook,       M_FOLDERHOOK},
    {"mbox-hook",           mutt_parse_hook,       M_MBOXHOOK},
    {"message-hook",        mutt_parse_hook,       M_MESSAGEHOOK},
    {"open-hook",           mutt_parse_hook,       M_OPENHOOK},
    {"pgp-hook",            mutt_parse_hook,       M_CRYPTHOOK},
    {"reply-hook",          mutt_parse_hook,       M_REPLYHOOK},
    {"save-hook",           mutt_parse_hook,       M_SAVEHOOK},
    {"send2-hook",          mutt_parse_hook,       M_SEND2HOOK},
    {"send-hook",           mutt_parse_hook,       M_SENDHOOK},
    {"macro",               mutt_parse_macro,      0},
    {"mono",                mutt_parse_mono,       0},
    {"push",                mutt_parse_push,       0},
    {"uncolor",             mutt_parse_uncolor,    0},
    {"unhook",              mutt_parse_unhook,     0},
    {"unmono",              mutt_parse_unmono,     0},
    {"alias",               parse_alias,           0},
    {"attachments",         parse_attachments,     0 },
    {"my_hdr",              parse_my_hdr,          0},
    {"reset",               parse_set,             M_SET_RESET},
    {"set",                 parse_set,             0},
    {"toggle",              parse_set,             M_SET_INV},
    {"unset",               parse_set,             M_SET_UNSET},
    {"source",              parse_source,          0},
    {"unalias",             parse_unalias,         0},
    {"unattachments",       parse_unattachments,   0},
    {"unmy_hdr",            parse_unmy_hdr,        0},
    {NULL, NULL, 0}
};
