/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 1999-2002 Thomas Roessler <roessler@does-not-exist.org>
 * Copyright (C) 2004 g10 Code GmbH
 *
 * Parts written/modified by:
 * Nico Golde <nion@muttng.org>
 * Andreas Krennmair <ak@synflood.at>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#define MAIN_C 1

#include <lib-lib/lib-lib.h>

#include <sys/utsname.h>

#include <lib-lua/lib-lua.h>
#include <lib-sys/mutt_signal.h>
#include <lib-mime/mime.h>
#include <lib-ui/lib-ui.h>
#include <lib-mx/mx.h>

#include "mutt.h"
#include "crypt.h"
#include "alias.h"
#include "buffy.h"
#include "sort.h"
#include "keymap.h"
#include "mutt_idna.h"
#include "mutt_sasl.h"

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
extern char *optarg;
extern int optind;
#endif

#ifdef HAVE_LIBIDN
#include <stringprep.h>
#endif

#ifdef USE_NNTP
#include "nntp.h"
#endif

#ifdef USE_HCACHE
#if defined(HAVE_QDBM)
#include <depot.h>
#elif defined(HAVE_GDBM)
#include <gdbm.h>
#endif
#endif

#include <gnutls/gnutls.h>
#include <gpgme.h>

void mutt_exit(int code)
{
    mutt_endwin(NULL);
    exit(code);
}

static void mutt_usage (void)
{
    puts(mutt_make_version());

    puts(_("\
usage: madmutt [ -nRyzZ ] [ -e <cmd> ] [ -F <file> ] [ -f <file> ]\n\
       madmutt [ -n ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
       madmutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
       madmutt -v\n"));

    puts(_("Options:"));
    puts(_("  -a <file>     attach a file to the message"));
    puts(_("  -b <address>  specify a blind carbon-copy (BCC) address"));
    puts(_("  -c <address>  specify a carbon-copy (CC) address"));
    puts(_("  -e <command>  specify a command to be executed after initialization"));
    puts(_("  -f <file>     specify which mailbox to read"));
    puts(_("  -F <file>     specify an alternate Madmuttrc file"));
#ifdef USE_NNTP
    puts(_("  -g <server>   specify a newsserver"));
    puts(_("  -G            select a newsgroup"));
#endif
    puts(_("  -H <file>     specify a draft file to read header and body from"));
    puts(_("  -i <file>     specify a file which Madmutt should include in the body"));
    puts(_("  -n            causes Madmutt not to read the system Madmuttrc"));
    puts(_("  -p            recall a postponed message"));
    puts(_("  -R            open mailbox in read-only mode"));
    puts(_("  -s <subj>     specify a subject (must be in quotes if it has spaces)"));
    puts(_("  -v            show version and compile-time definitions"));
    puts(_("  -y            select a mailbox specified in your `mailboxes' list"));
    puts(_("  -z            exit immediately if there are no messages in the mailbox"));
    puts(_("  -Z            open the first folder with new message, exit immediately if none"));
    puts(_("  -h            this help message"));

    exit(0);
}

static void show_version (void)
{
    struct utsname uts;
    uname(&uts);

    puts(mutt_make_version());
    puts(_("  Copyright (C) 1996-2002 Michael R. Elkins and others."));
    puts(_("  Copyright (C) 2005      The Mutt-ng Team"));
    puts(_("  Copyright (C) 2006-2007 Pierre Habouzit"));
    puts(_("  MadMutt is based on Mutt-ng wich was based on Mutt before"));
    puts("");

    printf("System:\n  %s %s (%s)\n", uts.sysname, uts.release, uts.machine);
    puts("External Libraries:");
#ifdef NCURSES_VERSION
    printf("  ncurses %s\n", NCURSES_VERSION);
#endif
#ifdef _LIBICONV_VERSION
    printf("  libiconv %d.%d\n", _LIBICONV_VERSION >> 8,
           _LIBICONV_VERSION & 0xff);
#endif
#ifdef STRINGPREP_VERSION
    printf("  libidn %s\n", STRINGPREP_VERSION);
#endif
#ifdef USE_HCACHE
#if defined(HAVE_QDBM)
    printf("  qdbm %s\n", dpversion);
#elif defined(HAVE_GDBM)
    printf("  gdbm %s\n", gdbm_version);
#endif
#endif
    printf("  gnutls %s\n", LIBGNUTLS_VERSION);
    printf("  gpgme %s\n",  GPGME_VERSION);
    puts (_("Compile Options:"));

    puts (
#ifdef USE_FCNTL
        "  +USE_FCNTL"
#else
        "  -USE_FCNTL"
#endif
#ifdef USE_FLOCK
        "  +USE_FLOCK"
#else
        "  -USE_FLOCK"
#endif
#ifdef USE_HCACHE
        "  +USE_HCACHE"
#else
        "  -USE_HCACHE"
#endif
#ifdef USE_NNTP
        "  +USE_NNTP"
#else
        "  -USE_NNTP"
#endif
#ifdef HAVE_LIBIDN
        "  +HAVE_LIBIDN"
#else
        "  -HAVE_LIBIDN"
#endif
        );

    puts(_("Built-In Defaults:"));
    printf("  SENDMAIL   \"%s\"\n", SENDMAIL);
    printf("  MAILPATH   \"%s\"\n", MAILPATH);
    printf("  PKGDATADIR \"%s\"\n", PKGDATADIR);
    printf("  PKGDOCDIR  \"%s\"\n", PKGDOCDIR);
    printf("  SYSCONFDIR \"%s\"\n", SYSCONFDIR);

    puts("");
    puts(_("This is free software.  You may redistribute copies of it under the terms of"));
    puts(_("the GNU General Public License <http://www.gnu.org/licenses/gpl.html>."));
    puts(_("There is NO WARRANTY, to the extent permitted by law."));

    exit(0);
}

static void start_curses (void)
{
  km_init ();                   /* must come before mutt_init */
  mutt_signal_initialize ();
  if (initscr () == NULL) {
    puts _("Error initializing terminal.");

    exit (1);
  }
  mutt_signal_initialize ();
  curses_initialize();
}

#define M_IGNORE  (1<<0)        /* -z */
#define M_BUFFY   (1<<1)        /* -Z */
#define M_NOSYSRC (1<<2)        /* -n */
#define M_RO      (1<<3)        /* -R */
#define M_SELECT  (1<<4)        /* -y */
#ifdef USE_NNTP
#define M_NEWS    (1<<5)        /* -g and -G */
#endif

__attribute__((format(printf, 1, 0)))
static void mutt_nocurses_error (const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fputc('\n', stderr);
}

int main (int argc, char **argv)
{
  char folder[_POSIX_PATH_MAX] = "";
  char *subject = NULL;
  char *includeFile = NULL;
  char *draftFile = NULL;
  HEADER *msg = NULL;
  string_list_t *attach = NULL;
  string_list_t *commands = NULL;
  int sendflags = 0;
  int flags = 0;
  int version = 0;
  int i;
  int explicit_folder = 0;

  /* initialize random number for tmp file creation */ 
  srand48((unsigned int) time (NULL));
  
  /* sanity check against stupid administrators */
  
  if (getegid () != getgid ()) {
    fprintf (stderr, "%s: I don't want to run with privileges!\n", argv[0]);
    exit (1);
  }

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, MUTTLOCALEDIR);
  textdomain (PACKAGE);
  setlocale (LC_CTYPE, "");

  mutt_error = mutt_message = mutt_nocurses_error;
  srand48 (time (NULL));
  umask (077);

  while ((i = getopt(argc, argv, "a:b:F:f:c:e:H:s:i:hnpRTtvyzZ"
#ifdef USE_NNTP
                                                                    "g:G"
#endif
                    )) >= 0)
    switch (i) {
    case 'a':
      if (strlen(optarg)<=512)
        attach = mutt_add_list (attach, optarg);
      else{
        printf("too long arguments. exiting ...\n");
        exit(1);
      }
      break;

    case 'F':
      m_strreplace(&Muttrc, optarg);
      break;

    case 'f':
      m_strcpy(folder, sizeof(folder), optarg);
      explicit_folder = 1;
      break;

    case 'b':
    case 'c':
      if (!msg)
        msg = header_new();
      if (!msg->env)
        msg->env = envelope_new();
      if (i == 'b')
        msg->env->bcc = rfc822_parse_adrlist (msg->env->bcc, optarg);
      else
        msg->env->cc = rfc822_parse_adrlist (msg->env->cc, optarg);
      break;

    case 'e':
      commands = mutt_add_list (commands, optarg);
      break;

    case 'H':
      draftFile = optarg;
      break;

    case 'i':
      includeFile = optarg;
      break;

    case 'n':
      flags |= M_NOSYSRC;
      break;

    case 'p':
      sendflags |= SENDPOSTPONED;
      break;

    case 'R':
      flags |= M_RO;            /* read-only mode */
      break;

    case 's':
      subject = optarg;
      break;

    case 'v':
      version++;
      break;

    case 'y':                  /* My special hack mode */
      flags |= M_SELECT;
      break;

#ifdef USE_NNTP
    case 'g':                  /* Specify a newsserver */
      {
        char buf[LONG_STRING];

        snprintf (buf, sizeof (buf), "set nntp_host=%s", optarg);
        commands = mutt_add_list (commands, buf);
      }

    case 'G':                  /* List of newsgroups */
      flags |= M_SELECT | M_NEWS;
      break;
#endif

    case 'z':
      flags |= M_IGNORE;
      break;

    case 'Z':
      flags |= M_BUFFY | M_IGNORE;
      break;

    default:
      mutt_usage ();
    }

  if (version) {
    show_version ();
  }

  /* Check for a batch send. */
  if (!isatty (0)) {
    set_option(OPTNOCURSES);
    sendflags = SENDBATCH;
  }

  /* This must come before mutt_init() because curses needs to be started
     before calling the init_pair() function to set the color scheme.  */
  if (!option (OPTNOCURSES))
    start_curses();

  /* set defaults and read init files */
  mutt_init (flags & M_NOSYSRC, commands);
  string_list_wipe(&commands);

  if (!option(OPTNOCURSES)) {
      ui_layout_init();
  }

  /* Create the Maildir directory if it doesn't exist. */
  if (!option(OPTNOCURSES) && Maildir) {
    struct stat sb;
    char fpath[_POSIX_PATH_MAX];
    char mesg[STRING];

    m_strcpy(fpath, sizeof(fpath), Maildir);
    mutt_expand_path (fpath, sizeof (fpath));
    /* we're not connected yet - skip mail folder creation */
    if (mx_get_magic (fpath) != M_IMAP)
      if (stat (fpath, &sb) == -1 && errno == ENOENT) {
        snprintf (mesg, sizeof (mesg), _("%s does not exist. Create it?"),
                  Maildir);
        if (mutt_yesorno (mesg, M_YES) == M_YES) {
          if (mkdir (fpath, 0700) == -1 && errno != EEXIST)
            mutt_error (_("Can't create %s: %s."), Maildir, strerror (errno));
        }
      }
  }

  if (sendflags & SENDPOSTPONED) {
    if (!option (OPTNOCURSES))
      mutt_flushinp ();
    ci_send_message (SENDPOSTPONED, NULL, NULL, NULL, NULL);
    mutt_endwin (NULL);
  }
  else if (subject || msg || sendflags || draftFile || includeFile || attach
           || optind < argc) {
    FILE *fin = NULL;
    char buf[LONG_STRING];
    char *tempfile = NULL, *infile = NULL;
    char *bodytext = NULL;
    FILE *fout;

    if (!option (OPTNOCURSES))
      mutt_flushinp ();

    if (!msg)
      msg = header_new();

    if (draftFile)
      infile = draftFile;
    else {
      if (!msg->env)
        msg->env = envelope_new();

      for (i = optind; i < argc; i++) {
        if (url_check_scheme (argv[i]) == U_MAILTO)
          url_parse_mailto (msg->env, &bodytext, argv[i]);
        else
          msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
      }

      if (option (OPTAUTOEDIT) && !msg->env->to && !msg->env->cc) {
        if (!option (OPTNOCURSES))
          mutt_endwin (NULL);
        fputs (_("No recipients specified.\n"), stderr);
        exit (1);
      }

      if (subject)
        msg->env->subject = m_strdup(subject);

      if (includeFile)
        infile = includeFile;
    }

    if (infile || bodytext) {
      if (infile) {
        if (m_strcmp("-", infile) == 0)
          fin = stdin;
        else {
          char path[_POSIX_PATH_MAX];

          m_strcpy(path, sizeof(path), infile);
          mutt_expand_path (path, sizeof (path));
          if ((fin = fopen (path, "r")) == NULL) {
            if (!option (OPTNOCURSES))
              mutt_endwin (NULL);
            perror (path);
            exit (1);
          }
        }
      }
      else
        fin = NULL;

      if (draftFile)
        msg->env = mutt_read_rfc822_header (fin, NULL, 1, 0);

      /* is the following if still needed? */

      fout = m_tempfile(buf, sizeof(buf), NONULL(mod_core.tmpdir), NULL);
      tempfile = m_strdup(buf);

      if (tempfile) {
        if (!fout) {
          if (!option (OPTNOCURSES))
            mutt_endwin (NULL);
          perror (tempfile);
          m_fclose(&fin);
          p_delete(&tempfile);
          exit (1);
        }
        if (fin)
          mutt_copy_stream (fin, fout);
        else if (bodytext)
          fputs (bodytext, fout);
        m_fclose(&fout);
        if (fin && fin != stdin)
          m_fclose(&fin);
      }
    }

    p_delete(&bodytext);

    if (attach) {
      string_list_t *t = attach;
      BODY *a = NULL;

      while (t) {
        if (a) {
          a->next = mutt_make_file_attach (t->data);
          a = a->next;
        }
        else
          msg->content = a = mutt_make_file_attach (t->data);
        if (!a) {
          if (!option (OPTNOCURSES))
            mutt_endwin (NULL);
          fprintf (stderr, _("%s: unable to attach file.\n"), t->data);
          string_list_wipe(&attach);
          exit (1);
        }
        t = t->next;
      }
      string_list_wipe(&attach);
    }

    ci_send_message (sendflags, msg, tempfile, NULL, NULL);

    if (!option (OPTNOCURSES))
      mutt_endwin (NULL);
  }
  else {
    if (flags & M_BUFFY) {
      if (!buffy_check (0)) {
        mutt_endwin _("No mailbox with new mail.");

        exit (1);
      }
      folder[0] = 0;
      buffy_next (folder, sizeof (folder));
    }
    else if (flags & M_SELECT) {
#ifdef USE_NNTP
      if (flags & M_NEWS) {
        set_option (OPTNEWS);
        if (!(CurrentNewsSrv = mutt_select_newsserver (NewsServer))) {
          mutt_endwin (Errorbuf);
          exit (1);
        }
      } else
#endif
      if (!Incoming.len) {
        mutt_endwin _("No incoming mailboxes defined.");

        exit (1);
      }
      folder[0] = 0;
      mutt_select_file(folder, sizeof(folder), M_SEL_FOLDER | M_SEL_BUFFY,
                       NULL, NULL);
      if (!folder[0]) {
        mutt_endwin (NULL);
        exit (0);
      }
    }

    if (!folder[0])
      m_strcpy(folder, sizeof(folder), NONULL(Spoolfile));

#ifdef USE_NNTP
    if (option (OPTNEWS)) {
      unset_option (OPTNEWS);
      nntp_expand_path (folder, sizeof (folder),
                        &CurrentNewsSrv->conn->account);
    } else
#endif
      mutt_expand_path (folder, sizeof (folder));

    m_strreplace(&CurrentFolder, folder);
    m_strreplace(&LastFolder, folder);

    if (flags & M_IGNORE) {
      /* check to see if there are any messages in the folder */
      switch (mx_check_empty (folder)) {
      case -1:
        mutt_endwin (strerror (errno));
        exit (1);
      case 1:
        mutt_endwin _("Mailbox is empty.");
        exit (1);
      }
    }

    mutt_folder_hook (folder);

    if ((Context = mx_open_mailbox(folder, (flags & M_RO) ?  M_READONLY : 0,
                                   NULL)) || !explicit_folder)
    {
      mutt_index_menu ();
      if (option (OPTXTERMSETTITLES))
        printf("\033]2;%s\007", NONULL(XtermLeave));
      if (Context)
        p_delete(&Context);
    }
    mutt_endwin (Errorbuf);
  }

  luaM_shutdown();
  mutt_sasl_shutdown();
  exit (0);
}
